#!/usr/local/bin/python
# -*- coding: utf-8 -*-

"""
This file is part of trnlp.

trnlp is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

trnlp is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with trnlp.  If not, see <https://www.gnu.org/licenses/>.
Copyright (c) 2016-2020, Esat Mahmut Bayol

The full license is in the file LICENSE.txt, distributed with this software.
"""
from trnlp.constant import allVowels, frontVowels, backVowels, roundedVowels, unroundedVowels, harmonyRule
from trnlp.cleaner import clean_quites
import itertools
import pickle
import bz2
import re


def levenshtein_distance(fword: str, sword: str) -> int:
    """
    Levenshtein distance algoritması iki kelimenin birbirlerine olan uzaklığını verir.
    :param fword: İlk kelime
    :param sword: İkinci kelime
    :return: int olarak Levenshtein uzaklığı
    """
    lenfword = len(fword)
    lensword = len(sword)
    minlen = min(lenfword, lensword)

    if fword == "":
        return lensword
    if sword == "":
        return lenfword

    difference = abs(lensword - lenfword)
    result = 0
    reciprocal = 0

    for i in range(1, minlen + 1):
        if fword[i - 1] == sword[i - 1]:
            result = result + 0
        else:
            result = result + 1

        if fword[-i] == sword[-i]:
            reciprocal = reciprocal + 0
        else:
            reciprocal = reciprocal + 1

    result = min(result, reciprocal)
    result = result + difference
    return result


def wordtoten(word: str) -> str:
    """
    Gelen kelimeyi, sesli harfler "1", sessiz harfler "0" olmak üzere sayıya çevirir.
    :param word: Döndürülecek kelime.
    :return: 0101... şeklnde döndürür.
    """
    word = to_lower(word)
    wtt = ""
    for ch in word:
        if ch in allVowels:
            wtt += '1'
        else:
            wtt += '0'
    return wtt


def to_lower(word: str) -> str:
    """
    Gelen string(yazı) veriyi küçük harfe çevirir.
    :param word: Küçük harfe çevrilecek string değer.
    :return: Küçük harfe çevrilmiş string değer.
    """
    tolower_text = (word.replace('İ', 'i'))
    tolower_text = (tolower_text.replace('I', 'ı'))
    return tolower_text.lower()


def to_upper(word: str) -> str:
    """
    Gelen string(yazı) veriyi büyük harfe çevirir.
    :param word: Büyük harfe çevrilecek string değer.
    :return: Büçük harfe çevrilmiş string değer.
    """
    toupper_text = (word.replace('i', 'İ'))
    toupper_text = (toupper_text.replace('ı', 'I'))
    return toupper_text.upper()


def capital_tr(word: str) -> str:
    """
    Gelen string(yazı) verisinin ilk harfini büyük harfe çevirir.
    :param word: İlk harfi büyük harfe çevrilecek string değer.
    :return: İlk harfi büyük harfe çevrilmiş string değer.
    """
    return '{}{}'.format(to_upper(word[0]), word[1:])


def isCap(word: str) -> bool:
    """
    İlk harfin büyük olup olmadığını kontrol eder.
    :param word: Kontrol edilecek string(yazı) verisi
    :return: True yada False döndürür.
    """
    try:
        return word[0].isupper()
    except IndexError:
        return False


def repc(word: str) -> str:
    """
    Şapkalı harfleri normale çevirir.
    :param word: str
    :return: str
    """
    replace_letter = [('â', 'a'), ('ê', 'e'), ('î', 'i'), ('ô', 'o'), ('û', 'u')]
    for tpl in replace_letter:
        word = word.replace(tpl[0], tpl[1])
    return word


def comb(x: list, y: list) -> list:
    """
    :param x : list
    :param y : list
    :return: İki listenin kombinasyonunu liste olarak döndürür.
    """
    return list(itertools.product(x, y))


def syllabification(word: str) -> list:
    """
    Heceleme algoritması
    :param word : Hecelenecek kelime
    :return: Heceleri liste içerisinde string olarak döndürür
    """
    spell_slicer = (('001000', 5), ('000100', 5), ('01000', 4), ('00100', 4), ('00010', 4), ('1000', 3), ('0100', 3),
                    ('0011', 3), ('0010', 3), ('011', 2), ('010', 2), ('100', 2), ('10', 1), ('11', 1))

    syllable_list = []
    tenword = wordtoten(word)

    if ('111' in tenword) or (tenword.endswith('000')):
        return []

    len_spell = tenword.count('1')

    if (len_spell == 1) or (tenword == '0'):
        return [word]

    for i in range(tenword.count('1')):
        for x, y in spell_slicer:
            if tenword.startswith(x):
                syllable_list.append(word[:y])
                word = word[y:]
                tenword = tenword[y:]
                break

    if tenword == '0':
        syllable_list[-1] = syllable_list[-1] + word
    elif word:
        syllable_list.append(word)

    if len(syllable_list) != len_spell:
        return []

    return syllable_list


def vowel_harmony(f_str: str) -> bool:
    """
    Genel sesli uyumu kontrolü yapar.
    :param f_str : Sesli uyumunun kontrol edileceği kelime string şeklinde
    :return: bool
    """
    result = True

    cln_quites = clean_quites(f_str) + '.'

    if cln_quites == '.':
        return result

    for i, letter in enumerate(cln_quites):
        if cln_quites[i + 1] == '.' or letter == '.':
            break
        elif cln_quites[i + 1] in harmonyRule[letter]:
            continue
        else:
            result = False
    return result


def labial_harmony(word: str) -> bool:
    """
    Küçük sesli uyumu kontrolü yapar. True yada False döndürür.
    :param word: str
    :return: bool
    """
    cln_word = clean_quites(to_lower(repc(word)))

    if (not cln_word) or (len(cln_word) == 1):
        return True

    first_char = cln_word[0]
    for char in cln_word[1:]:
        second_char = char
        if (first_char in unroundedVowels) and (second_char in unroundedVowels):
            first_char = second_char
            continue
        elif (first_char in roundedVowels) and (second_char in 'aeuü'):
            first_char = second_char
            continue
        else:
            return False

    return True


def palatal_harmony(word: str) -> bool:
    """
    Büyük sesli uyumu kontrolü yapar. True yada False döndürür.
    :param word: str
    :return: bool
    """
    cln_word = clean_quites(to_lower(repc(word)))

    if (not cln_word) or (len(cln_word) == 1):
        return True

    first_char = cln_word[0]
    for char in cln_word[1:]:
        second_char = char
        if (first_char in frontVowels) and (second_char in frontVowels):
            first_char = second_char
            continue
        elif (first_char in backVowels) and (second_char in backVowels):
            first_char = second_char
            continue
        else:
            return False

    return True


def number_to_word(number: str) -> str:
    """
    Sayıyı yazıya dönüştürür.
    :param number: Dönüştürülece sayı string şeklinde
    :return: Yazıya döndürülmüş sayı string şeklinde
    """

    def three_digits_number(co, ii):
        trk_number_3 = {0: '',
                        1: 'bin',
                        2: 'milyon',
                        3: 'milyar',
                        4: 'trilyon',
                        5: 'katrilyon'}
        _number_list = []

        if (ii == 1) and (co == '001'):
            return 'bin'

        for i, _number in enumerate(co):
            if _number == '0':
                continue

            if i == 0:
                if _number == '1':
                    _number_list.append('yüz')
                else:
                    _number_list = _number_list + [trk_numbers_0_9[_number], 'yüz']
            elif i == 1:
                _number_list.append(trk_numbers_10_90[_number])
            elif i == 2:
                _number_list.append(trk_numbers_0_9[_number])
        return ' '.join(_number_list) + ' ' + trk_number_3[ii]

    if ',' in number:
        splt_number = number.split(',')
        fp = number_to_word(splt_number[0])
        sp = number_to_word(splt_number[1])
        return fp + 'virgül ' + sp

    trk_numbers_0_9 = {'1': 'bir',
                       '2': 'iki',
                       '3': 'üç',
                       '4': 'dört',
                       '5': 'beş',
                       '6': 'altı',
                       '7': 'yedi',
                       '8': 'sekiz',
                       '9': 'dokuz',
                       '0': ''}
    trk_numbers_10_90 = {'1': 'on',
                         '2': 'yirmi',
                         '3': 'otuz',
                         '4': 'kırk',
                         '5': 'elli',
                         '6': 'altmış',
                         '7': 'yetmiş',
                         '8': 'seksen',
                         '9': 'doksan',
                         '0': ''}

    number = number.replace('.', '')
    len_number = len(number)
    if len_number % 3 == 0:
        add_zero = 0
    else:
        add_zero = 3 - (len_number % 3)

    number = ('0' * add_zero) + number
    number_list = [number[i:i + 3] for i in range(0, len(number), 3)]
    number_list.reverse()

    ntow = ''
    for s, part in enumerate(number_list):
        if part == '000':
            continue
        ntow = three_digits_number(part, s) + ' ' + ntow

    if not ntow:
        return 'sıfır'
    else:
        return ntow


def word_to_number(number_string: str) -> str:
    """
    Yazıyla yazılmış olan sayıyı sayıya dönüştürür.
    :param number_string: Dönüştürülece yazı string şeklinde
    :return: Sayıya döndürülmüş yazı string şeklinde
    """

    def three_digits_number(co):
        yuzluk = '000'
        reco = [i for i in re.split(regexi, co) if i]
        if reco:
            reco.reverse()
            reco = reco + ['']
            for i, it in enumerate(reco):
                if it == 'yüz':
                    if reco[i + 1] in trk_numbers_0_9:
                        yuzluk = trk_numbers_0_9[reco[i + 1]] + yuzluk[1:]
                    else:
                        yuzluk = '1' + yuzluk[1:]
                    break
                elif it in trk_numbers_10_90:
                    yuzluk = yuzluk[0] + trk_numbers_10_90[it] + yuzluk[2]
                elif it in trk_numbers_0_9:
                    yuzluk = yuzluk[:2] + trk_numbers_0_9[it]
        return yuzluk

    if number_string.strip() == 'sıfır':
        return '0'

    if 'virgül' in number_string:
        splt_word = number_string.split('virgül')
        fp = word_to_number(splt_word[0])
        sp = word_to_number(splt_word[1])
        return fp + ',' + sp

    trk_numbers_0_9 = {'bir'  : '1',
                       'iki'  : '2',
                       'üç'   : '3',
                       'dört' : '4',
                       'beş'  : '5',
                       'altı' : '6',
                       'yedi' : '7',
                       'sekiz': '8',
                       'dokuz': '9'}

    trk_numbers_10_90 = {'on'    : '1',
                         'yirmi' : '2',
                         'otuz'  : '3',
                         'kırk'  : '4',
                         'elli'  : '5',
                         'altmış': '6',
                         'yetmiş': '7',
                         'seksen': '8',
                         'doksan': '9'}

    cd = {'katrilyon': 18, 'trilyon': 15, 'milyar': 12, 'milyon': 9, 'bin': 6, 'yüz': 3}
    number_digit = ['katrilyon', 'trilyon', 'milyar', 'milyon', 'bin']
    regexi = '(yüz|doksan|seksen|yetmiş|altmış|elli|kırk|otuz|yirmi|on|dokuz|sekiz|yedi|altı|beş|dört|üç|iki|bir)'

    _word = number_string.replace(' ', '')
    liste = []
    uzunluk = 0
    for part in number_digit:
        repart = re.search(part, _word)
        if repart:
            x, y = repart.span()
            _uzunluk = cd[_word[x:y]]
            if _uzunluk > uzunluk:
                uzunluk = _uzunluk
            liste.append((_word[:x], _word[x:y]))
            _word = _word[y:]

    if _word:
        liste.append(_word)

    sayi = '0' * uzunluk

    for part in liste:
        if type(part) is tuple:
            if part == ('', 'bin'):
                sayi = sayi[:-cd[part[1]]] + '001' + sayi[3 - cd[part[1]]:]
                continue
            sayi = sayi[:-cd[part[1]]] + three_digits_number(part[0]) + sayi[3 - cd[part[1]]:]
        elif type(part) is str:
            sayi = sayi[:-3] + three_digits_number(part)
    sayi = sayi.lstrip('0')

    return sayi


def nth_replace(string, old, new, n=1) -> str:
    """
    Eşleşen n'inci karakteri yenisi ile değiştirir.
    :param string: string
    :param old: Değiştirilecek karakter
    :param new: Yerine yazılacak karakter
    :param n: Eşleşen kaçıncı karakterin değiştirileceği
    :return: string
    """
    left_join = old
    right_join = old
    groups = string.split(old)
    nth_split = [left_join.join(groups[:n]), right_join.join(groups[n:])]
    return new.join(nth_split)


def package_path():
    return __file__[:-9].replace('\\', '/')


def tr_sort(liste: list):
    """
    Liste içerisindeki string değişkenleri Türk alfabesine göre sıralar.
    :param liste: Liste içinde string değişkenler
    :return: Sıralanmış liste
    """
    tr_letters = "abcçdefgğhıijklmnoöprsştuüvyz"
    trans = {i: tr_letters.index(i) for i in tr_letters}
    return sorted(liste, key=lambda x: trans.get(x[0]))


def change_punch(satir):
    rep_text = satir
    char_rep_dict = {'”'   : '"',
                     '“'   : '"',
                     '’'   : "'",
                     '‘'   : "'",
                     "`"   : "'",
                     '…'   : '...',
                     'ș'   : 'ş',
                     '""'  : '"',
                     '\'\'': '"'}

    for old, new in char_rep_dict.items():
        rep_text = rep_text.replace(old, new)
    return rep_text


def repa(word):
    apostrophes = "ʿ’'`.-"
    for apos in apostrophes:
        word = word.replace(apos, '')
    return word


def n_gram(n_iter, n=2) -> list:
    ngram_list = []

    if len(n_iter) == n:
        ngram_list.append(n_iter)
    elif len(n_iter) < n:
        return []
    else:
        for i, char in enumerate(n_iter):
            if i < len(n_iter) - (n - 1):
                ngram_list.append(n_iter[i:i + n])

    return ngram_list


def debug(func):
    import functools

    @functools.wraps(func)
    def wrapper_debug(*args, **kwargs):
        args_repr = [repr(a) for a in args]
        kwargs_repr = [f"{k}={v!r}" for k, v in kwargs.items()]
        signature = ", ".join(args_repr + kwargs_repr)
        print(f"Çağırdı {func.__name__}({signature})")
        value = func(*args, **kwargs)
        print(f"{func.__name__!r} döndürdü {value!r}")
        return value

    return wrapper_debug


def unrepeated_list(l: list) -> list:
    temp = []
    for x in l:
        if x not in temp:
            temp.append(x)
    return temp


def compressed_pickle(title, data):
    with bz2.BZ2File(title + ".pbz2", "w") as f:
        pickle.dump(data, f, protocol=pickle.HIGHEST_PROTOCOL)


def decompress_pickle(file):
    data = bz2.BZ2File(file, "rb")
    data = pickle.load(data)
    return data


if __name__ == '__main__':
    with open(package_path() + 'data/freq_dict.pickle', 'rb') as handle:
        lexicon = pickle.load(handle)

    compressed_pickle("freq_dict", lexicon)



#!/usr/local/bin/python
# -*- coding: utf-8 -*-

"""
This file is part of trnlp.

trnlp is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

trnlp is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with trnlp.  If not, see <https://www.gnu.org/licenses/>.
Copyright (c) 2016-2020, Esat Mahmut Bayol

The full license is in the file LICENSE.txt, distributed with this software.
"""

# r"(ocak|şubat|mart|nisan|mayıs|haziran|temmuz|ağustos|eylül|ekim|kasım|aralık)"

abbr = [r"(?=\b([a-zçğıİöşü]+\.\s[a-zçğıİöşü]+\.\s[a-zçğıİöşü]+\.)(?=\s))",
        r"(?=\b([a-zçğıİöşü]+\.\s[a-zçğıİöşü]+\.)(?=\s))",
        r"(?=\b([a-zçğıİöşü]+-[a-zçğıİöşü]+)(?=\s))",
        r"(?=\b([a-zçğıİöşü]+\.)(?=\s))"]

date_rgx = [r"(?:\b(0[1-9]|1[0-9]|2[0-9]|3[0-1])\s"
            r"([eoşmnhtak][kcuaiİeğyr][abrsyzmuliİ][a-zıü]{1,4})"
            r"(\s\d{2,4})?)",
            r"(?:\b(0[1-9]|1[0-9]|2[0-9]|3[01])[/.-](0[1-9]|1[0-2])[/.-]\d{2,4})",
            r"(\d{1,4})(?= yılı)"]

fin_rgx = [r"(?:[\$€₺]\d+(\.\d\d\d)*(,\d\d)?)",
           r"(?:\d+(\.\d\d\d)*\s?(try|türk l[iİ]rası|tl|₺|l[iİ]ra|avro|euro|dolar))",
           r"(?:(%|‰|yüzde|b[iİ]nde)\s?\d+(,\d+)?)",
           r"(?:\d+(,\d+)?(%|‰))"]

time_rgx = [r"(?:(0?[0-9]|1[0-9]|[0-2][0-4])[.:][0-5][0-9])",
            r"(?:(0?[0-9]|1[0-2])([.:][0-5][0-9])?\s?(am|pm|a\.m\.|p\.m\.))"]

phone_rgx = [r"(?:(\+9|009)?0?\s?(5[0-5][0-9])\s?\d{3}\s?\d{2}\s?\d{2})",
             r"(?:(\+9|009)?0?\s?([234][0-9][0-9])\s?\d{3}\s?\d{2}\s?\d{2})"]

web_rgx = [r"(?:[a-z0-9_\-\.]+)@([a-z0-9_\-\.]+)\.([a-z]{2,5})",
           r"(?:(https?://)?(www\.)?[a-z0-9]+\.[a-z]{3,6}[.a-z0-9\?=\-\+%&\_/]*)"]

th_rgx = [r"(?<=\s)(\d+\.)(?=\s)",
          r"(?<=\s)([xvı]+\.)(?=\s)", ]

nbr_rgx = [r"(?:(\d+)([,.]\d+)?)",
           r"(?:(\d{1,3})([,.]\d{3})*([,.]\d+)?)"]

enter_regex = [r"\n"]

space_regex = [r"\s"]

proper_noun_abbr = [r"((?<=\s)([a-zçğıİöşü]\.))"]

words = [r"([0-9a-zÀ-ž]+([’'][a-zçğıöşü]+)?)"]

suff = [r"(?<=')([a-zÀ-ž]+)"]

end_of_seq = [r"(\?\.\.)|(!\.\.)|(\.\.\.)|(\?)|(!)|(:)|(\.)"]

punch = [r"[^\w\s]"]

all_regex = [(fin_rgx, 'financial'),
             (date_rgx, 'date'),
             (time_rgx, 'time'),
             (phone_rgx, 'phone'),
             (web_rgx, 'web'),
             (th_rgx, 'th'),
             (nbr_rgx, 'number'),
             (proper_noun_abbr, 'proabbr'),
             (end_of_seq, 'eos'),
             (words, 'word'),
             (suff, 'suffix'),
             (enter_regex, 'enter'),
             (space_regex, 'space'),
             (punch, 'punch')]



# -*- coding: utf-8 -*-

"""
Example usage:
my_ascii_turkish_txt = "Opusmegi cagristiran catirtilar."
deasciifier = Deasciifier(my_ascii_turkish_txt)
my_deasciified_turkish_txt = deasciifier.convert_to_turkish()
print(my_deasciified_turkish_txt)
This system is based on the turkish-mode by Dr. Deniz Yüret
Python Code by Emre Sevinç
"""

import pickle
from trnlp.helper import package_path

__all__ = ['Deasciifier']

ascii_pickle_path = package_path() + 'data/ascii_to_str_dict.pickle'


class Deasciifier:
    with open(ascii_pickle_path, 'rb') as f:
        turkish_pattern_table = pickle.load(f)
    del f

    turkish_context_size = 10

    turkish_asciify_table = {u'ç': u'c',
                             u'Ç': u'C',
                             u'ğ': u'g',
                             u'Ğ': u'G',
                             u'ö': u'o',
                             u'Ö': u'O',
                             u'ü': u'u',
                             u'Ü': u'U',
                             u'ı': u'i',
                             u'İ': u'I',
                             u'ş': u's',
                             u'Ş': u'S'}

    turkish_downcase_asciify_table = {'H': 'h', 'C': 'c', 'm': 'm', 'ı': 'i', 'q': 'q', 'İ': 'i', 'N': 'n', 'U': 'u',
                                      'Ç': 'c', 'i': 'i', 'G': 'g', 'I': 'i', 'w': 'w', 'k': 'k', 'o': 'o', 'a': 'a',
                                      'D': 'd', 'p': 'p', 'Ö': 'o', 'W': 'w', 'd': 'd', 'S': 's', 'Q': 'q', 'Y': 'y',
                                      't': 't', 'A': 'a', 'l': 'l', 'f': 'f', 'ç': 'c', 'P': 'p', 'J': 'j', 's': 's',
                                      'b': 'b', 'ü': 'u', 'r': 'r', 'x': 'x', 'B': 'b', 'v': 'v', 'j': 'j', 'K': 'k',
                                      'h': 'h', 'F': 'f', 'Ü': 'u', 'T': 't', 'Ğ': 'g', 'ş': 's', 'z': 'z', 'g': 'g',
                                      'V': 'v', 'n': 'n', 'ö': 'o', 'O': 'o', 'ğ': 'g', 'c': 'c', 'R': 'r', 'y': 'y',
                                      'Ş': 's', 'M': 'm', 'E': 'e', 'u': 'u', 'X': 'x', 'Z': 'z', 'e': 'e', 'L': 'l'}

    turkish_upcase_accents_table = {'E': 'e', 'Ü': 'U', 'ğ': 'G', 'X': 'x', 'ü': 'U', 'o': 'o', 'M': 'm', 'Ç': 'C',
                                    'm': 'm', 'L': 'l', 'u': 'u', 's': 's', 'H': 'h', 'C': 'c', 'y': 'y', 'T': 't',
                                    'V': 'v', 'N': 'n', 'B': 'b', 'j': 'j', 'Ğ': 'G', 'J': 'j', 'x': 'x', 'v': 'v',
                                    'O': 'o', 'A': 'a', 'F': 'f', 'Q': 'q', 'w': 'w', 'r': 'r', 'ı': 'I', 'Z': 'z',
                                    'p': 'p', 'U': 'u', 'P': 'p', 't': 't', 'g': 'g', 'ç': 'C', 'Y': 'y', 'Ş': 'S',
                                    'I': 'i', 'e': 'e', 'h': 'h', 'İ': 'i', 'z': 'z', 'd': 'd', 'K': 'k', 'k': 'k',
                                    'l': 'l', 'f': 'f', 'G': 'g', 'n': 'n', 'D': 'd', 'c': 'c', 'i': 'i', 'ş': 'S',
                                    'W': 'w', 'q': 'q', 'R': 'r', 'Ö': 'O', 'S': 's', 'ö': 'O', 'b': 'b', 'a': 'a'}

    def __init__(self, ascii_string):
        self.ascii_string = ascii_string
        self.turkish_string = ascii_string

    def print_turkish_string(self):
        print(self.turkish_string)

    @staticmethod
    def set_char_at(mystr, pos, c):
        return mystr[0:pos] + c + mystr[pos + 1:]

    def convert_to_turkish(self):
        # Convert a string with ASCII-only letters into one with Turkish letters.
        for i in range(len(self.turkish_string)):
            c = self.turkish_string[i]
            if self.turkish_need_correction(c, point=i):
                self.turkish_string = self.set_char_at(self.turkish_string, i, self.turkish_toggle_accent(c))
            else:
                self.turkish_string = self.set_char_at(self.turkish_string, i, c)

        return self.turkish_string

    @staticmethod
    def turkish_toggle_accent(c):
        turkish_toggle_accent_table = {
                u'c': u'ç',
                u'C': u'Ç',
                u'g': u'ğ',
                u'G': u'Ğ',
                u'o': u'ö',
                u'O': u'Ö',
                u'u': u'ü',
                u'U': u'Ü',
                u'i': u'ı',
                u'I': u'İ',
                u's': u'ş',
                u'S': u'Ş',
                u'ç': u'c',
                u'Ç': u'C',
                u'ğ': u'g',
                u'Ğ': u'G',
                u'ö': u'o',
                u'Ö': u'O',
                u'ü': u'u',
                u'Ü': u'U',
                u'ı': u'i',
                u'İ': u'I',
                u'ş': u's',
                u'Ş': u'S'
        }
        return turkish_toggle_accent_table.get(c, c)

    def turkish_need_correction(self, char, point=0):
        # Determine if char at cursor needs correction.
        ch = char
        tr = Deasciifier.turkish_asciify_table.get(ch, ch)
        pl = Deasciifier.turkish_pattern_table.get(tr.lower(), False)

        if pl:
            m = self.turkish_match_pattern(pl, point)
        else:
            m = False

        if tr == u'I':
            if ch == tr:
                return not m
            else:
                return m
        else:
            if ch == tr:
                return m
            else:
                return not m

    def turkish_match_pattern(self, dlist, point=0):
        # Check if the pattern is in the pattern table.
        rank = 2 * len(dlist)
        str_char = self.turkish_get_context(Deasciifier.turkish_context_size, point=point)
        start = 0

        _len = len(str_char)
        while start <= Deasciifier.turkish_context_size:
            end = 1 + Deasciifier.turkish_context_size
            while end <= _len:
                s = str_char[start:end]
                r = dlist.get(s, False)
                if r and abs(r) < abs(rank):
                    rank = r
                end = 1 + end
            start = 1 + start

        return rank > 0

    def turkish_get_context(self, size=turkish_context_size, point=0):
        s = ' ' * (1 + (2 * size))
        s = s[0:size] + 'X' + s[size + 1:]
        i = 1 + size
        space = False
        index = point
        index = index + 1

        while i < len(s) and not space and index < len(self.ascii_string):
            current_char = self.turkish_string[index]
            x = Deasciifier.turkish_downcase_asciify_table.get(current_char, False)
            if not x:
                if not space:
                    i = i + 1
                    space = True
            else:
                s = s[0:i] + x + s[i + 1:]
                i = i + 1
                space = False
            index = index + 1

        s = s[0:i]

        index = point
        i = size - 1
        space = False

        index = index - 1
        while i >= 0 and index >= 0:
            current_char = self.turkish_string[index]
            x = Deasciifier.turkish_upcase_accents_table.get(current_char, False)
            if not x:
                if not space:
                    i = i - 1
                    space = True
            else:
                s = s[0:i] + x + s[i + 1:]
                i = i - 1
                space = False
            index = index - 1

        return s


if __name__ == '__main__':
    my_ascii_turkish_txt = "Opusmegi cagristiran catirtilar."
    deasciifier = Deasciifier(my_ascii_turkish_txt)
    my_deasciified_turkish_txt = deasciifier.convert_to_turkish()
    print(my_deasciified_turkish_txt)



#!/usr/local/bin/python
# -*- coding: utf-8 -*-

"""
This file is part of trnlp.

trnlp is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

trnlp is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with trnlp.  If not, see <https://www.gnu.org/licenses/>.
Copyright (c) 2016-2020, Esat Mahmut Bayol

The full license is in the file LICENSE.txt, distributed with this software.
"""

from trnlp.morphology import TrnlpWord
from itertools import product, groupby
from trnlp.constant import *
from trnlp.helper import *
from trnlp.tokenization import word_token

__all__ = ['SpellingCorrector']


class SpellingCorrector:
    __freq_dict = {}
    _obj_ = TrnlpWord()

    def __init__(self):
        self.__org_word = ""
        self.__word_list = ""
        self.__load_inf_dict()
        self.__iteration = False

    def settext(self, word: str) -> None:
        self.__word_list = word_token(to_lower(word))

    @classmethod
    def __load_inf_dict(cls) -> None:
        cls.__freq_dict = decompress_pickle(package_path() + 'data/freq_dict.pbz2')

    def __split(self, word: str) -> list:
        return [(word[:i], word[i:]) for i in range(len(word) + 1)]

    def __word_calculator(self, word: str) -> bool:
        counter = 0
        for letter in to_lower(word):
            if letter in allLowerLetters:
                counter += 1
        if (counter / len(word)) > 0.65:
            return True

    def vowelizero(self, word: str) -> list:
        if not self.__word_calculator(word):
            return []

        a1 = ['', 'a', 'e', 'ı', 'i', 'o', 'ö', 'u', 'ü']

        if word[0] in allVowels:
            result = [word[0]]
        else:
            result = [a1, word[0]]

        for i in range(1, len(word)):
            fl = word[i - 1]
            fs = word[i]

            if (fl not in allVowels) and (fs not in allVowels):
                result.append(a1)
                result.append(fs)
            else:
                result.append(fs)

        if word[-1] not in allVowels:
            result.append(a1)

        return [''.join(x) for x in product(*result) if ''.join(x) in self.__freq_dict]

    def deasciifier(self, word: str) -> list:
        ascii_list = []
        for char in word:
            if char in deascii_dict:
                ascii_list.append(deascii_dict[char])
            else:
                ascii_list.append([char])

        result = ['']
        for may in ascii_list:
            result = [''.join(s) for s in product(result, may)]
        return result

    def unrepeater(self, word: str) -> list:
        result = []
        i1 = [c for c, _ in groupby(word)]
        i2 = [''.join(c)[:2] for _, c in groupby(word)]
        zip_i = [set(i) for i in list(zip(i1, i2))]

        a = zip_i[0]

        for i in zip_i[1:]:
            result = [''.join(s) for s in product(a, i)]
            a = result
        return result

    def transposes(self, word: str) -> list:
        split_word = self.__split(word)
        return [L + R[1] + R[0] + R[2:] for L, R in split_word if len(R) > 1]

    def inserts(self, word: str, char_set=trLowerLetters) -> list:
        split_word = self.__split(word)
        return [L + c + R for L, R in split_word for c in char_set]

    def replaces(self, word: str, char_set=trLowerLetters) -> list:
        split_word = self.__split(word)
        return [L + c + R[1:] for L, R in split_word if R for c in char_set]

    def deletes(self, word: str) -> list:
        split_word = self.__split(word)
        return [L + R[1:] for L, R in split_word if R]

    def __known(self, words: list) -> list:
        known_list = []
        for w in words:
            if w in self.__freq_dict:
                known_list.append(w)
        return known_list

    def __freq(self, x1):
        f1 = self.__freq_dict[x1[0]]
        if x1[1] <= 0:
            return f1 * 2
        else:
            return f1 / x1[1]

    def __manipulate(self, word, **kwargs):
        result = set()

        for key, value in kwargs.items():
            if (key == "deasciifier") and (value is True):
                [result.add((x, 0.1)) for x in self.__known(self.deasciifier(word))]
            elif (key == "unrepeater") and (value is True):
                [result.add((x, (len(self.__org_word) - len(x))))
                 for x in self.__known(self.unrepeater(word))]
            elif (key == "transposes") and (value is True):
                [result.add((x, 2)) for x in self.__known(self.transposes(word))]
            elif (key == "inserts") and (value is True):
                [result.add((x, 0.5)) for x in self.__known(self.inserts(word))]
            elif (key == "replaces") and (value is True):
                [result.add((x, 2)) for x in self.__known(self.replaces(word))]
            elif (key == "deletes") and (value is True):
                [result.add((x, 1)) for x in self.__known(self.deletes(word))]
            elif (key == "vowelizero") and (value is True) and (self.__iteration is False):
                [result.add((x, (len(x) - len(self.__org_word)))) for x in self.vowelizero(word)]

        return list(result)

    def correction(self, **kwargs):
        result = []

        if not self.__word_list:
            return [[self.__org_word]]

        if "all" not in kwargs:
            kwargs["all"] = False

        if "word_list" in kwargs:
            _wordlist = kwargs["word_list"]
        else:
            _wordlist = self.__word_list

        if kwargs["all"] is True:
            kwargs["deasciifier"] = True
            kwargs["unrepeater"] = True
            kwargs["transposes"] = True
            kwargs["inserts"] = True
            kwargs["replaces"] = True
            kwargs["deletes"] = True
            kwargs["vowelizero"] = True

        for word in _wordlist:
            self.__org_word = word
            temp = self.__manipulate(word, **kwargs)

            if temp:
                temp.sort(key=lambda x: self.__freq(x))
                temp.reverse()

            self._obj_.setword(word)
            inf_temp = []
            if self._obj_.get_inf:
                for may in self._obj_.get_inf:
                    inf_temp.append((self._obj_.correct_form(may),))

            if inf_temp:
                temp = inf_temp + temp
            temp = unrepeated_list([x[0] for x in temp])

            if not temp:
                result.append([word])
            else:
                result.append(temp)

        return result


if __name__ == '__main__':
    pass



#!/usr/local/bin/python
# -*- coding: utf-8 -*-

"""
This file is part of trnlp.

trnlp is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

trnlp is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with trnlp.  If not, see <https://www.gnu.org/licenses/>.
Copyright (c) 2016-2020, Esat Mahmut Bayol

The full license is in the file LICENSE.txt, distributed with this software
"""

from re import findall
from trnlp.constant import allVowels


def rfind_parenthesize(x: str):
    """
    Harf dizisinin en sonunda bulunan '{}' süslü parantez içerisindeki ifadeyi döndürür.
    :param x : str
    :return: str
    """
    return x[x.rfind('(') + 1:x.rfind(')')]


def rfind_curly_braces(x: str) -> str:
    """
    Harf dizisinin en sonunda bulunan '{}' süslü parantez içerisindeki ifadeyi döndürür.
    :param x : str
    :return: str
    """
    return x[x.rfind('{') + 1:x.rfind('}')]


def rfind_square_bracket(x: str) -> str:
    """
    Harf dizisinin en sonunda bulunan '[]' köşeli parantez içerisindeki ifadeyi döndürür.
    :param x : str
    :return: str
    """
    return x[x.rfind('[') + 1:x.rfind(']')]


def find_table_number(adding: str) -> int:
    """
    Ekin tablo numarasını döndürür.
    :param adding : str
    :return: int
    """
    numbers = rfind_square_bracket(adding)
    list_no = numbers[:numbers.find('_')]
    return int(list_no)


def find_row_number(adding: str) -> int:
    """
    Ekin tablo içerisindeki satır numarasını döndürür.
    :param adding : str
    :return: int
    """
    numbers = rfind_square_bracket(adding)
    row_no = numbers[numbers.find('_') + 1:]
    return int(row_no)


def find_numbers(inf: str):
    items_no = findall(r'\[(.*?)\]', inf)
    return [(int(x.split('_')[0]), int(x.split('_')[1])) for x in items_no if x]


def base_finder(string):
    return string[:string.find('(')]


def last_vowel(word: str) -> str:
    """
    Kelimenin son sesli harfini bulur.
    :param word: str
    :return: char
    """
    vowel = ''
    i = 1
    while i <= len(word):
        if word[-i] in allVowels:
            vowel = word[-i]
            break
        i += 1
    return vowel


def first_vowel(word: str) -> str:
    """
    Kelimenin ilk sesli harfini bulur.
    :param word: str
    :return: char
    """
    vowel = ''
    i = 0
    while i < len(word):
        if word[i] in allVowels:
            vowel = word[i]
            break
        i += 1
    return vowel



#!/usr/local/bin/python
# -*- coding: utf-8 -*-

"""
This file is part of trnlp.

trnlp is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

trnlp is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with trnlp.  If not, see <https://www.gnu.org/licenses/>.
Copyright (c) 2016-2020, Esat Mahmut Bayol

The full license is in the file LICENSE.txt, distributed with this software.
"""

from trnlp.helper import package_path, to_lower, isCap
from trnlp.constant import stopwords
from trnlp.token_data.data import *
from pickle import load
import collections
import re

__all__ = ['TrnlpToken',
           'unitoascii',
           'simple_token',
           'whitespace_token',
           'word_token']


def _decode(string: str) -> str:
    """
    Bu fonksiyon unidecode kütüphanesinden alınmış ve türkçe için özelleştirilmiştir.
    Türkçe için genişletilmiş Ascii çevirici.
    :param string: unicode string
    :return: Tr Ascii string
    """
    cache = {}
    decval = []

    for char in string:
        codepoint = ord(char)

        if codepoint in {199, 214, 220, 231, 246, 252, 286, 287, 304, 305, 350, 351, 8364, 8378, 8240}:
            decval.append(str(char))
            continue

        if codepoint < 0x80:
            decval.append(str(char))
            continue

        if codepoint > 0xeffff:
            continue

        section = codepoint >> 8
        position = codepoint % 256

        if section in cache:
            table = cache[section]
        else:
            try:
                mod = __import__('unidecoder.x%03x' % section, fromlist=['data'])
            except ImportError:
                cache[section] = None
                continue
            cache[section] = table = mod.data

        if table and len(table) > position:
            decval.append(table[position])

    return ''.join(decval)


def unitoascii(string: str) -> str:
    """
    Türkçe için genişletilmiş Ascii çevirici.
    :param string: unicode string
    :return: Tr Ascii string
    """
    try:
        string.encode('ASCII')
    except UnicodeEncodeError:
        return _decode(string)

    return string


def simple_token(utext: str, sw=None) -> list:
    """
    Yazı içerisindeki her karakteri parçalar
    :param utext: Parçalanacak yazı
    :param sw:
    :return: liste şeklinde parçalanmış yazı ve karakterler
    """
    if sw is None:
        sw = ()
    result = list(filter(None, [i.strip() for i in re.split(r"(\W+?\s?)", utext)]))
    return [x for x in result if to_lower(x) not in sw]


def whitespace_token(utext: str, sw=None) -> list:
    """
    Yazıyı boşluklara göre parçalar
    :param utext: Parçalanacak yazı
    :param sw:
    :return: liste şeklinde parçalanmış yazı
    """
    if sw is None:
        sw = ()
    result = [x for x in utext.split(' ') if x]
    return [x for x in result if to_lower(x) not in sw]


def word_token(utext: str, numbers=True, sw=None) -> list:
    """
    Yazı içerisindeki kelimeleri döndürür. Sayılar yada sayı içeren bölümler bulunmaz.
    :param utext: Parçalanacak yazı
    :param numbers:
    :param sw:
    :return: liste şeklinde kelimeler
    """
    if sw is None:
        sw = ()
    try:
        # Gerekli karakter dönüşümleri
        utext = unitoascii(utext).replace("''", '"')

        # Boş satırları temizle
        utext = '\n'.join([line for line in utext.split('\n') if line.strip()])

        # Birden fazla olan boşlukları teke düşür.
        utext = re.sub(r" +", " ", utext)

        a1 = []
        a2 = ""
        for line in utext.split('\n'):
            line = line.strip()
            if line.endswith('-'):
                a2 = a2 + line[:-1]
                continue
            else:
                if a2:
                    splt_line = line.split(' ')
                    a2 = a2 + splt_line[0]
                    a1.append(a2)
                    a1.append(' '.join(splt_line[1:]))
                    a2 = ""
                else:
                    a1.append(line)
        if a2:
            a1.append(a2)

        utext = '\n'.join(a1)

        if numbers:
            regexi = r"([a-z0-9À-ž]+([’\'][a-z]+)?)"
        else:
            regexi = r"([a-zÀ-ž]+([’\'][a-z]+)?)"

        result = [x[0] for x in re.findall(regexi, utext, flags=re.IGNORECASE)]
        return [x for x in result if to_lower(x) not in sw]
    except IndexError:
        return []


class TrnlpToken:
    def __init__(self):
        self.__text = ""
        self.__spans = []
        self.__types = []
        self.__tokens = []
        self.__shortList = {}
        self.__load_dict()

    @property
    def spans(self) -> list:
        return self.__spans

    @property
    def types(self) -> list:
        return self.__types

    @property
    def tokens(self) -> list:
        return self.__tokens

    @property
    def wordtoken(self) -> list:
        return self.__word_token()

    @property
    def phrasetoken(self) -> list:
        return self.__phrase_token()

    @property
    def wordcounter(self):
        return self.__word_counter()

    @property
    def ziptoken(self) -> list:
        return self.__zip_token()

    def settext(self, text) -> None:
        self.__spans = []
        self.__types = []
        self.__tokens = []
        self.__text = text
        self.__arr_text()
        self.__span_token()

    def __load_dict(self) -> None:
        with open(package_path() + 'data/shortlist.pickle', 'rb') as handle:
            self.__shortList = load(handle)

    def __arr_text(self) -> None:
        # Gerekli karakter dönüşümleri
        self.text = unitoascii(self.__text).replace("''", '"')

        # Boş satırları temizle
        self.__text = '\n'.join([line for line in self.__text.split('\n') if line.strip()])

        # Birden fazla olan boşlukları teke düşür.
        self.__text = re.sub(r" +", " ", self.__text)

        a1 = []
        a2 = ""
        for line in self.__text.split('\n'):
            line = line.strip()
            if line.endswith('-'):
                a2 = a2 + line[:-1]
                continue
            else:
                if a2:
                    splt_line = line.split(' ')
                    a2 = a2 + splt_line[0]
                    a1.append(a2)
                    a1.append(' '.join(splt_line[1:]))
                    a2 = ""
                else:
                    a1.append(line)
        if a2:
            a1.append(a2)
        self.__text = '\n'.join(a1)

    def __span_token(self) -> None:
        self.__abbr_decoder()

    def __regex_finder(self, regex_list: list):
        for regex in regex_list:
            for matches in re.finditer(regex, self.__text, flags=re.IGNORECASE | re.DOTALL):
                yield matches

    def __abbr_decoder(self) -> None:
        for matches in self.__regex_finder(abbr):
            for grp_num in range(0, len(matches.groups())):
                grp_num = grp_num + 1
                match = to_lower(matches.group(grp_num))
                if match in self.__shortList:
                    self.__spans.append(matches.span(grp_num))
                    self.__types.append('abbr')
        self.__regex_decoder()

    def __regex_decoder(self) -> None:
        non_eol = set()
        for matches in self.__regex_finder([r"([\[\(\"].*?[\"\)\]])"]):
            x, y = matches.span()
            [non_eol.add(x) for x in range(x, y)]

        for regex_list, r_types in all_regex:
            for matches in self.__regex_finder(regex_list):
                x, y = matches.span()
                if r_types == 'word':
                    mword = to_lower(self.__text[x:y])
                    if mword in self.__shortList:
                        self.__spans.append(matches.span())
                        self.__types.append('abbr')
                        continue
                elif r_types == 'eos':
                    if x in non_eol:
                        self.__spans.append(matches.span())
                        self.__types.append('punch')
                        continue
                self.__spans.append(matches.span())
                self.__types.append(r_types)
        self.__span_solver()

    def __span_solver(self) -> None:
        span_set = set()
        spans = []
        types = []
        for s in range(len(self.__spans)):
            span_type = self.__types[s]
            x, y = self.__spans[s]
            if x == y:
                continue
            elif (y - x == 1) and (x not in span_set):
                spans.append((x, y))
                types.append(span_type)
                span_set.add(x)
                continue
            else:
                in_span_set = [i for i in range(x, y) if i in span_set]
                if not in_span_set:
                    spans.append((x, y))
                    types.append(span_type)
                    [span_set.add(x) for x in range(x, y)]

        spans_type = list(zip(spans, types))
        spans_type.sort(key=lambda l: l[0][0])
        self.__spans = [x for x, y in spans_type]
        self.__types = [y for x, y in spans_type]
        self.__set_tokens()

    def __set_tokens(self) -> None:
        self.__tokens = [self.__text[self.__spans[i][0]:self.__spans[i][1]] for i in range(len(self.__spans))]

    @staticmethod
    def is_header(txt: str) -> bool:
        txt = txt.split(' ')
        txt_list = [True for wrd in txt if isCap(wrd)]
        return True if (len(txt_list) / len(txt)) > 0.65 else False

    def __word_token(self) -> list:
        return [self.__tokens[i] for i in range(len(self.__tokens))
                if (self.__types[i] == 'word') or (self.__types[i] == 'suffix')]

    def __zip_token(self) -> list:
        return list(zip(self.__tokens, self.__types, self.__spans))

    def __phrase_token(self) -> list:
        phrase_list = []
        phrase = ""
        for tkn, tkn_type in zip(self.__tokens, self.__types):
            if tkn_type == 'eos':
                phrase = "{}{}".format(phrase, tkn)
                phrase_list.append(phrase.strip())
                phrase = ""
            elif tkn_type == 'enter':
                phrase = "{}{}".format(phrase, ' ')
            else:
                phrase = "{}{}".format(phrase, tkn)
        if phrase:
            phrase_list.append(phrase.strip())
        return phrase_list

    def __word_counter(self):
        return collections.Counter(self.__word_token())

    def clean_punch(self, zip_list=None) -> list:
        punchs = {'eos', 'enter', 'space', 'punch'}
        if zip_list is None:
            zip_list = self.__zip_token()
        result = []
        for part, wtype, span in zip_list:
            if wtype in punchs:
                continue
            else:
                result.append((part, wtype, span))
        return result

    def clean_stopwords(self, zip_list=None) -> list:
        if zip_list is None:
            zip_list = self.__zip_token()
        result = []
        for part, wtype, span in zip_list:
            if to_lower(part) in stopwords:
                continue
            else:
                result.append((part, wtype, span))
        return result


if __name__ == '__main__':
    pass



#!/usr/local/bin/python
# -*- coding: utf-8 -*-

"""
This file is part of trnlp.

trnlp is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

trnlp is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with trnlp.  If not, see <https://www.gnu.org/licenses/>.
Copyright (c) 2016-2020, Esat Mahmut Bayol

The full license is in the file LICENSE.txt, distributed with this software.
"""

puncs = {'!' : 'ecphoneme',
         '"' : 'double-quote',
         '“' : 'double-quote',
         '”' : 'double-quote',
         '#' : 'pound',
         '$' : 'dollar',
         '€' : 'euro',
         '%' : 'percent',
         '&' : 'and',
         '\'': 'apostrophe',
         '’' : 'apostrophe',
         '(' : 'open parenthesis',
         ')' : 'close parenthesis',
         '*' : 'asterisk',
         '+' : 'plus',
         ',' : 'comma',
         '-' : 'minus',
         '.' : 'dot',
         '/' : 'slash',
         ':' : 'colon',
         ';' : 'semi-colon',
         '<' : 'less than',
         '=' : 'equal',
         '>' : 'greater than',
         '?' : 'question mark',
         '@' : 'at',
         '[' : 'open bracket',
         '\\': 'back slash',
         ']' : 'close bracket',
         '^' : 'carat',
         '_' : 'underscore',
         '`' : 'back quote',
         '{' : 'open curly braces',
         '|' : 'pipe',
         '}' : 'close curly braces',
         '~' : 'tilde'}



#!/usr/local/bin/python
# -*- coding: utf-8 -*-

"""
This file is part of trnlp.

trnlp is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

trnlp is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with trnlp.  If not, see <https://www.gnu.org/licenses/>.
Copyright (c) 2016-2020, Esat Mahmut Bayol

The full license is in the file LICENSE.txt, distributed with this software.
"""

from trnlp.helper import *
from trnlp.controler import ssc
from itertools import product
from trnlp.data.suffix_tables import *
import os
import re

save_folder_path = package_path() + 'data/'


def delete_folder():
    sfx_files = ['dafNoun.pickle', 'dafVerb.pickle', 'infVerb.pickle', 'infNoun.pickle']
    folder_list = os.listdir(save_folder_path)
    for folder_name in folder_list:
        if folder_name in sfx_files:
            os.unlink(save_folder_path + folder_name)
            print(save_folder_path + folder_name + ' dosyası silindi.')


def sfx_vowel_harmony(suffix: str, sfx_prop: tuple) -> bool:
    """
    :param suffix : Sesli uyumunun kontrol edileceği ek
    :param sfx_prop : Sesli uyumunun kontrol edileceği ekin özellik dizisi
    :return: 'True' ya da 'False' döndürür
    """
    if (7 in sfx_prop) or (vowel_harmony(suffix)):
        return True
    return False


def regex_solver(sfx_regex: str, sfx_prop: tuple) -> list:
    """
    :param sfx_regex : Ek tablosundaki ekin regex ifadesi
    :param sfx_prop : Ek tablosundaki ekin özellikler dizisi
    :return: Tüm ihtimalleri liste olarak döndürür
    """
    b = list(re.finditer(r"\w|\[(\w+)\]", sfx_regex))
    result = list()
    for match in b:
        if '[' not in match.group():
            result.append(match.group())
        else:
            result.append(list(match.group()[1:-1]))
    return [''.join(x) for x in product(*result) if sfx_vowel_harmony(''.join(x), sfx_prop)]


def suffix_suffix_control(suffix_list, suffix_table, suffix_types, suffix_prop):
    temp_dict = {'suffixes'   : suffix_list,
                 'suffixPlace': suffix_table,
                 'suffixProp' : suffix_prop,
                 'suffixTypes': suffix_types}
    if ssc(temp_dict, 0):
        return True
    return False


def enclisis(f_sfx_var_list: list):
    for f_sfx in f_sfx_var_list:
        f_table_number = f_sfx[1][-1][0]
        f_row_number = f_sfx[1][-1][1]
        f_after_sfx_nos = st_dict[f_table_number][f_row_number][4]
        for f_after_sfx_no in f_after_sfx_nos:
            if f_after_sfx_no == -1:
                continue
            if f_after_sfx_no == 0:
                continue
            s_sfx = st_dict[f_table_number][f_after_sfx_no]
            s_sfx_var_list = regex_solver(s_sfx[3], s_sfx[5])
            s_sfx_result = [([ek], [(f_table_number, f_after_sfx_no)], [s_sfx[2]], [s_sfx[5]]) for ek in s_sfx_var_list]
            adding = list(f_sfx)
            adding = [(adding[0] + ekl, adding[1] + positionl, adding[2] + typel, adding[3] + propl)
                      for ekl, positionl, typel, propl in s_sfx_result]
            [f_sfx_var_list.append(add) for add in adding if suffix_suffix_control(add[0], add[1], add[2], add[3])]

    return f_sfx_var_list


def crt(table_str, table_no):
    temp_dict = dict()
    for row_no, row in enumerate(eval(table_str), 0):
        if row_no == 0:
            continue
        if -1 in row[4]:
            harmonic_list = regex_solver(row[3], row[5])
            harmonic_list = [([ek], [(table_no, row_no)], [row[2]], [row[5]]) for ek in harmonic_list]
            for enc_dict in enclisis(harmonic_list):
                clean_ekli = ''.join(enc_dict[0])
                if clean_ekli in temp_dict:
                    temp_dict[clean_ekli].append(enc_dict)
                else:
                    temp_dict[clean_ekli] = [enc_dict]
    return temp_dict


def save_to_file(table_str, dicter):
    if table_str == 'ice':
        table_str = 'infNoun'
    elif table_str == 'fce':
        table_str = 'infVerb'
    compressed_pickle(save_folder_path + table_str, dicter)


def create_suffix_pickle():
    delete_folder()
    daf_noun_dict = dict()
    daf_verb_dict = dict()

    for table_str in st_str_dict:
        print(table_str, 'tablosunun çekimleri yapılıyor...')
        table_no = st_str_dict[table_str]
        result_dict = crt(table_str, table_no)
        if table_no in {1, 2}:
            save_to_file(table_str, result_dict)
        elif table_no in {3, 6, 7, 8, 9}:
            for tmp in result_dict:
                if tmp in daf_verb_dict:
                    daf_verb_dict[tmp] = daf_verb_dict[tmp] + result_dict[tmp]
                else:
                    daf_verb_dict[tmp] = result_dict[tmp]
            continue
        elif table_no in {4, 5}:
            for tmp in result_dict:
                if tmp in daf_noun_dict:
                    daf_noun_dict[tmp] = daf_noun_dict[tmp] + result_dict[tmp]
                else:
                    daf_noun_dict[tmp] = result_dict[tmp]
            continue

    save_to_file('dafVerb', daf_verb_dict)
    save_to_file('dafNoun', daf_noun_dict)


if __name__ == '__main__':
    create_suffix_pickle()



#!/usr/local/bin/python
# -*- coding: utf-8 -*-

"""
This file is part of trnlp.

trnlp is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

trnlp is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with trnlp.  If not, see <https://www.gnu.org/licenses/>.
Copyright (c) 2016-2020, Esat Mahmut Bayol

The full license is in the file LICENSE.txt, distributed with this software.
"""

allLowerLetters = ('a', 'b', 'c', 'ç', 'd', 'e', 'f', 'g', 'ğ', 'h', 'ı', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'ö',
                   'p', 'r', 's', 'ş', 't', 'u', 'ü', 'v', 'y', 'z', 'q', 'w', 'x')

allUpperLetters = ('A', 'B', 'C', 'Ç', 'D', 'E', 'F', 'G', 'Ğ', 'H', 'I', 'İ', 'J', 'K', 'L', 'M', 'N', 'O', 'Ö',
                   'P', 'R', 'S', 'Ş', 'T', 'U', 'Ü', 'V', 'Y', 'Z', 'Q', 'W', 'X')

trLowerLetters = ('a', 'b', 'c', 'ç', 'd', 'e', 'f', 'g', 'ğ', 'h', 'ı', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'ö',
                  'p', 'r', 's', 'ş', 't', 'u', 'ü', 'v', 'y', 'z', 'â', 'ê', 'î', 'ô', 'û')

trUpperLetters = ('A', 'B', 'C', 'Ç', 'D', 'E', 'F', 'G', 'Ğ', 'H', 'I', 'İ', 'J', 'K', 'L', 'M', 'N', 'O', 'Ö',
                  'P', 'R', 'S', 'Ş', 'T', 'U', 'Ü', 'V', 'Y', 'Z')

allVowels = ('a', 'e', 'ı', 'i', 'o', 'ö', 'u', 'ü', 'â', 'ê', 'î', 'ô', 'û')

lowerVowels = ('a', 'e', 'ı', 'i', 'o', 'ö', 'u', 'ü')

backVowels = ('a', 'ı', 'o', 'u')

frontVowels = ('e', 'i', 'ö', 'ü')

unroundedVowels = ('a', 'e', 'ı', 'i')

roundedVowels = ('o', 'ö', 'u', 'ü')

capVowels = ('â', 'ê', 'î', 'ô', 'û')

fortis = {'f', 'h', 's', 'ç', 'ş', 'p', 't', 'k'}

prefix_lenis = {'c', 'd', 'g'}

lenis = {'b', 'c', 'd', 'g', 'ğ'}

deascii_dict = {'c': ['c', 'ç'],
                'g': ['g', 'ğ'],
                'i': ['ı', 'i'],
                'ı': ['ı', 'i'],
                'o': ['o', 'ö'],
                's': ['s', 'ş'],
                'u': ['u', 'ü']}

asciify_table = {u'ç': u'c',
                 u'Ç': u'C',
                 u'ğ': u'g',
                 u'Ğ': u'G',
                 u'ö': u'o',
                 u'Ö': u'O',
                 u'ü': u'u',
                 u'Ü': u'U',
                 u'ı': u'i',
                 u'İ': u'I',
                 u'ş': u's',
                 u'Ş': u'S'}

# inc_harmony_rule sözlüğü sesli uyumunu bozan kökün son ünlüsü ile
# eklenen ekin ilk ünlüsü arasındaki ilişkiyi belirler.
incHarmonyRule = {'a': 'ei',
                  'â': 'ei',
                  'i': 'aı',
                  'o': 'eü',
                  'u': 'eü',
                  'ü': 'au',
                  'û': 'eü'}

# harmony_rule sözlüğü kökün son ünlüsü ile eklenen ekin ilk ünlüsü arasındaki ilişkiyi belirler.
harmonyRule = {'a': 'aı',
               'â': 'aı',
               'e': 'ei',
               'ı': 'aı',
               'i': 'ei',
               'î': 'ei',
               'o': 'au',
               'ö': 'eü',
               'u': 'au',
               'û': 'au',
               'ü': 'eü'}

qKeybordNear = {'q': ('w', 's', 'a'),
                'w': ('e', 'd', 's', 'a', 'q'),
                'e': ('r', 'f', 'd', 's', 'w'),
                'r': ('t', 'g', 'f', 'd', 'e'),
                't': ('y', 'h', 'g', 'f', 'r'),
                'y': ('u', 'j', 'h', 'g', 't'),
                'u': ('ı', 'k', 'j', 'h', 'y'),
                'ı': ('o', 'l', 'k', 'j', 'u'),
                'o': ('p', 'ş', 'l', 'k', 'ı'),
                'p': ('ğ', 'i', 'ş', 'l', 'o'),
                'ğ': ('ü', 'i', 'ş', 'p'),
                'ü': ('i', 'ğ'),
                'a': ('q', 'w', 's', 'x', 'z'),
                's': ('q', 'w', 'e', 'd', 'c', 'x', 'z', 'a'),
                'd': ('w', 'e', 'r', 'f', 'v', 'c', 'x', 's'),
                'f': ('e', 'r', 't', 'g', 'b', 'v', 'c', 'd'),
                'g': ('r', 't', 'y', 'h', 'n', 'b', 'v', 'f'),
                'h': ('t', 'y', 'u', 'j', 'm', 'n', 'b', 'g'),
                'j': ('y', 'u', 'ı', 'k', 'ö', 'm', 'n', 'h'),
                'k': ('u', 'ı', 'o', 'l', 'ç', 'ö', 'm', 'j'),
                'l': ('ı', 'o', 'p', 'ş', 'ç', 'ö', 'k'),
                'ş': ('o', 'p', 'ğ', 'i', 'ç', 'l'),
                'i': ('p', 'ğ', 'ü', 'ş'),
                'z': ('a', 's', 'x',),
                'x': ('z', 'a', 's', 'd', 'c'),
                'c': ('x', 's', 'd', 'f', 'v'),
                'v': ('c', 'd', 'f', 'g', 'b'),
                'b': ('v', 'f', 'g', 'h', 'n'),
                'n': ('b', 'g', 'h', 'j', 'm'),
                'm': ('n', 'h', 'j', 'k', 'ö'),
                'ö': ('m', 'j', 'k', 'l', 'ç'),
                'ç': ('ö', 'k', 'l', 'ş')}

stopwords = {'acaba', 'ama', 'ancak', 'artık', 'asla', 'aslında', 'az', 'bana', 'bazen', 'bazı', 'bazıları',
             'bazısı', 'belki', 'ben', 'beni', 'benim', 'beş', 'bile', 'bir', 'biri', 'birisi', 'birkaç', 'birkaçı',
             'birçok', 'birçokları', 'birçoğu', 'birşey', 'birşeyi', 'biz', 'bize', 'bizi', 'bizim', 'bu', 'buna',
             'bunda', 'bundan', 'bunu', 'bunun', 'burada', 'böyle', 'böylece', 'bütün', 'da', 'daha', 'de', 'demek',
             'değil', 'diye', 'diğer', 'diğeri', 'diğerleri', 'dolayı', 'elbette', 'en', 'fakat', 'falan', 'felan',
             'filan', 'gene', 'gibi', 'hangi', 'hangisi', 'hani', 'hatta', 'hem', 'henüz', 'hep', 'hepsi',
             'hepsine', 'hepsini', 'her', 'herbiri', 'herkes', 'herkese', 'herkesi', 'hiç', 'hiçbiri', 'hiçbirine',
             'hiçbirini', 'hiçkimse', 'ile', 'ise', 'için', 'içinde', 'işte', 'kadar', 'kaç', 'kendi', 'kendine',
             'kendini', 'ki', 'kim', 'kime', 'kimi', 'kimin', 'kimisi', 'madem', 'mi', 'mu', 'mü', 'mı', 'nasıl',
             'ne', 'neden', 'nedir', 'nerde', 'nerede', 'nereden', 'nereye', 'nesi', 'neyse', 'niye', 'niçin',
             'ona', 'ondan', 'onlar', 'onlara', 'onlardan', 'onların', 'onu', 'onun', 'orada', 'oysa', 'oysaki',
             'sana', 'sen', 'senden', 'seni', 'senin', 'siz', 'sizden', 'size', 'sizi', 'sizin', 'son', 'sonra',
             'tabi', 'tamam', 'tüm', 'tümü', 'var', 've', 'veya', 'veyahut', 'ya', 'yada', 'yani', 'yerine', 'yine',
             'yoksa', 'zaten', 'zira', 'çok', 'çoğu', 'çoğuna', 'çoğunu', 'çünkü', 'öbürü', 'ön', 'önce', 'ötürü',
             'öyle', 'üzere', 'şayet', 'şey', 'şimdi', 'şu', 'şuna', 'şunda', 'şundan', 'şunlar', 'şunu', 'şunun',
             'şöyle'}



#!/usr/local/bin/python
# -*- coding: utf-8 -*-

"""
This file is part of trnlp.

trnlp is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

trnlp is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with trnlp.  If not, see <https://www.gnu.org/licenses/>.
Copyright (c) 2016-2020, Esat Mahmut Bayol

The full license is in the file LICENSE.txt, distributed with this software.
"""

from trnlp.deasciifier.Asciidecoder import *
from trnlp.spelling_corrector import *
from trnlp.file_prossesing import *
from trnlp.tokenization import *
from trnlp.morphology import *
from trnlp.constant import *
from trnlp.helper import *



#!/usr/local/bin/python
# -*- coding: utf-8 -*-

"""
This file is part of trnlp.

trnlp is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

trnlp is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with trnlp.  If not, see <https://www.gnu.org/licenses/>.
Copyright (c) 2016-2020, Esat Mahmut Bayol

The full license is in the file LICENSE.txt, distributed with this software.
"""

from trnlp.data.suffix_tables import *
from trnlp.controler import *
from trnlp.helper import *

__all__ = ['BaseFinder',
           'Derivational',
           'Inflections',
           'TrnlpWord',
           'writeable']


def writeable(inf_dict: dict, long=False) -> str:
    """
    Ek bulma fonksiyonu ile üretilen sözlük eklerin kısaltma isimleri kullanılarak yazdırılır.
    :param long: True olması durumunda, kısaltma isimler yerine uzun açıklamalar kullanılır
    :param inf_dict: Ek bulma fonksiyonu ile üretilen sözlük
    :return: arka(isim,sıfat)+daş{İi}[4_26]+lar{EfKe3ç}[1_52]
    """
    if not inf_dict:
        return ''

    if long:
        inf_dict['suffixTypes'] = [SUFFIX_SHORTLIST[x] for x in inf_dict['suffixTypes']]

    fadding = "{}({})".format(inf_dict['verifiedBase'], ','.join(inf_dict['baseType']))
    zip1 = zip(inf_dict['suffixes'], inf_dict['suffixTypes'])
    zip1 = [a + '{' + b + '}' for a, b in zip1]
    sfx_plc = ['[' + '_'.join((str(a), str(b))) + ']' for a, b in inf_dict['suffixPlace']]
    zip2 = list(zip(zip1, sfx_plc))

    if not zip2:
        return fadding
    elif len(zip2) > 1:
        return "{}+{}".format(fadding, '+'.join([a + b for a, b in zip2]))
    else:
        return "{}+{}{}".format(fadding, zip2[0][0], zip2[0][1])


class BaseFinder:
    _mainDict = {}  # Ana Sözlük
    _pronDict = {}  # Özel İsim Sözlüğü
    _abbrDict = {}  # Kısaltmalar Sözlüğü

    def __init__(self):
        self.__useMain = True  # Kök aramada ana sözlük kullanılsın mı?
        self.__usePron = True  # Kök aramada özel isimler sözlüğü kullanılsın mı?
        self.__useAbbr = False  # Kök aramada kısaltmalar sözlüğü kullanılsın mı?
        self._orgWord = ""  # Girilen kelimenin orjinal hali
        self._word = ""  # Girilen kelimenin işlem görmüş hali
        self.__load_lexicons()

    def __dir__(self):
        return self.__dict__.keys()

    @property
    def usemain(self) -> bool:
        return self.__useMain

    @property
    def usepron(self) -> bool:
        return self.__usePron

    @property
    def useabbr(self) -> bool:
        return self.__useAbbr

    @usemain.setter
    def usemain(self, isuse: bool):
        self.__useMain = isuse

    @usepron.setter
    def usepron(self, isuse: bool):
        self.__usePron = isuse

    @useabbr.setter
    def useabbr(self, isuse: bool):
        self.__useAbbr = isuse

    @classmethod
    def __load_lexicons(cls):
        if not cls._mainDict:
            cls._mainDict = decompress_pickle(package_path() + 'data/gen_tr_lex.pbz2')
        if not cls._pronDict:
            cls._pronDict = decompress_pickle(package_path() + 'data/prop_tr_lex.pbz2')
        if not cls._abbrDict:
            cls._abbrDict = decompress_pickle(package_path() + 'data/short_tr_lex.pbz2')

    def setword(self, string_word: str):
        self._orgWord = change_punch(string_word.strip())
        self._word = repa(repc(to_lower(string_word.strip())))

    @staticmethod
    def _splitter(string: str) -> list:
        """
        Kelimeyi parçalar. Örneğin 'deneme' -> ['deneme', 'denem', 'dene', 'den', 'de', 'd']
        :param string: Kök bulmak için parçalanacak string veri
        :return: Liste içerisinde parçalanmış string değerler döndürür.
        """
        result = [string[:x] for x in range(1, len(string) + 1)]
        result.reverse()
        return result

    def pos_bases(self) -> list:
        """
        'self.word' değişkeni ile tanımlanan kelimenin olası köklerini bulur.
        :return: Liste içerisinde sözlük(dict) olarak döndürür.
        """
        if any((self.__useMain, self.__useAbbr, self.__usePron)):
            return self.__find_in_lexicon()
        else:
            self.__useMain = True
            return self.__find_in_lexicon()

    def __find_in_lexicon(self) -> list:
        """
        Parçalanmış kelime listesindeki her bir kelimeyi genel sözlük(cls._mainDict) içerisinde arar.
        :return: Sözlük içerisinde bulunan sonuçları liste halinde döndürür.
        """
        org = to_lower(self._orgWord)
        splited_string = self._splitter(self._word)
        result = []

        if "." in self._orgWord:
            pword = re.sub(r"\W", "", org[:org.rfind(".")])
            residual = org[org.rfind(".") + 1:]
            if self.useabbr and (pword in self._abbrDict):
                [result.append((a2, residual)) for a2 in self._abbrDict[pword]]
        elif "'" in self._orgWord:
            pword = re.sub(r"\W", "", org[:org.rfind("'")])
            residual = org[org.rfind("'") + 1:]
            if residual and self.usepron and (pword in self._pronDict):
                [result.append((a2, residual)) for a2 in self._pronDict[pword]]
            if self.useabbr and (pword in self._abbrDict):
                [result.append((a2, residual)) for a2 in self._abbrDict[pword]]
            if word_to_number(pword):
                result.append(({'base'        : pword,
                                'verifiedBase': pword,
                                'baseType'    : ['isim'],
                                'baseProp'    : [''],
                                'etymon'      : 'Türkçe',
                                'event'       : 0,
                                'currentType' : ['isim'],
                                'purview'     : '0'}, residual))
        elif isCap(self._orgWord):
            if self.usepron:
                [result.append((a2, None))
                 for a1 in splited_string if a1 in self._pronDict for a2 in self._pronDict[a1]]
            if self.__useMain:
                [result.append((a2, None))
                 for a1 in splited_string if a1 in self._mainDict for a2 in self._mainDict[a1]]
            if self.useabbr:
                [result.append((a2, None))
                 for a1 in splited_string if a1 in self._abbrDict for a2 in self._abbrDict[a1]]
            if word_to_number(self._word):
                result.append(({'base'        : self._word,
                                'verifiedBase': self._word,
                                'baseType'    : ['isim'],
                                'baseProp'    : [''],
                                'etymon'      : 'Türkçe',
                                'event'       : 0,
                                'currentType' : ['isim'],
                                'purview'     : '0'}, None))
        else:
            if self.__useMain:
                [result.append((a2, None))
                 for a1 in splited_string if a1 in self._mainDict for a2 in self._mainDict[a1]]
            if word_to_number(self._word):
                result.append(({'base'        : self._word,
                                'verifiedBase': self._word,
                                'baseType'    : ['isim'],
                                'baseProp'    : [''],
                                'etymon'      : 'Türkçe',
                                'event'       : 0,
                                'currentType' : ['isim'],
                                'purview'     : '0'}, None))
            if self.usepron:
                [result.append((a2, None))
                 for a1 in splited_string if a1 in self._pronDict for a2 in self._pronDict[a1]]
            if self.useabbr:
                [result.append((a2, None))
                 for a1 in splited_string if a1 in self._abbrDict for a2 in self._abbrDict[a1]]

        return [self.__update_dict(a1) for a1 in result]

    def __update_dict(self, tpl: tuple) -> dict:
        """
        Bulunan kök sözlüğündeki bilinmeyen(None) değerleri doldurur ve geri döndürür.
        :param tpl: Bulunan kök sözlüğü ve kalan ek yada None
        :return: Kök sözlüğünün düzenlenmiş halini döndürür.
        """
        dl = tpl[0]
        residual = tpl[1]

        dl['orgWord'] = self._orgWord
        dl['word'] = self._word
        dl['suffixes'] = []
        dl['suffixPlace'] = []
        dl['suffixTypes'] = []
        dl['suffixProp'] = []

        if residual is None:
            ext_base = re.sub(r"\W", "", dl['base'])
            dl['residual'] = self._word[len(ext_base):]
        else:
            dl['residual'] = residual

        return dl


class Derivational(BaseFinder):
    _dafVerbDict = {}  # Fiile eklenebilecek yapım ekleri sözlüğü
    _dafNounDict = {}  # İsme eklenebilecek yapım ekleri sözlüğü

    def __init__(self):
        super().__init__()
        self.__load_daf_dict()

    def __dir__(self):
        return self.__dict__.keys()

    @classmethod
    def __load_daf_dict(cls):
        if not cls._dafVerbDict:
            cls._dafVerbDict = decompress_pickle(package_path() + 'data/dafVerb.pbz2')
        if not cls._dafNounDict:
            cls._dafNounDict = decompress_pickle(package_path() + 'data/dafNoun.pbz2')

    def _dfn(self, abc: str) -> list:
        """
        Arta kalan eki parçalar ve her bir parçayı isme eklenebilen yapım ekleri(_dafNoun)
        sözlüğü içerisinde arar ve sonucu döndürür.
        :param abc: aranacak arta kalan ek
        :return: sözlükte bulunan sonuçları liste halinde döndürür.
        """
        return [a2 for a1 in self._splitter(abc) if a1 in self._dafNounDict for a2 in self._dafNounDict[a1]]

    def _dfv(self, abc: str) -> list:
        """
        Arta kalan eki parçalar ve her bir parçayı fiile eklenebilen yapım ekleri(_dafVerb)
        sözlüğü içerisinde arar ve sonucu döndürür.
        :param abc: aranacak arta kalan ek
        :return: sözlükte bulunan sonuçları liste halinde döndürür.
        """
        return [a2 for a1 in self._splitter(abc) if a1 in self._dafVerbDict for a2 in self._dafVerbDict[a1]]

    @staticmethod
    def _der_type(sfx_place: list, sfx_types: list) -> list:
        """
        Kelimeye eklenen eklerin kelimeyi dönüştürdüğü kelime tipini bulur.
        :param sfx_place: Eklerin tablo ve satır numaralarının listesi
        :param sfx_types: Eklerin tanımları
        :return: Liste içerisinde kelime tiplerini döndürür.
        """
        result = []
        for i, place in enumerate(sfx_place, 0):
            if (place[0] == 2) and (sfx_types[i] in {'Olz', 'Ytsz'}):
                result.append('fiil')
            else:
                result.append(daf_type[sfx_place[i][0]])
        return result

    def derivational_lr(self) -> list:
        """
        Soldan sağa kelimeye eklenmiş olan olası yapım eklerini arar.
        :return: Liste içerisinde sözlük olarak sonuç döndürür.
        """
        main_dict_list = self.pos_bases()

        if not main_dict_list:
            return []

        for main_dict in main_dict_list:
            if not main_dict['residual']:
                continue

            if self._abbrDict and (main_dict['baseType'] == ['kısaltma']):
                continue

            if 'fiil' in main_dict['currentType'][-1]:
                pos_der = self._dfv(main_dict['residual'])
            else:
                pos_der = self._dfn(main_dict['residual'])

            for piece in pos_der:
                temp_dict = dict(main_dict)
                add_type = self._der_type(piece[1], piece[2])
                temp_dict['currentType'] = temp_dict['currentType'] + add_type
                temp_dict['suffixes'] = main_dict['suffixes'] + piece[0]
                temp_dict['suffixPlace'] = main_dict['suffixPlace'] + piece[1]
                temp_dict['suffixTypes'] = main_dict['suffixTypes'] + piece[2]
                temp_dict['suffixProp'] = main_dict['suffixProp'] + piece[3]
                temp_dict['residual'] = temp_dict['residual'][len(''.join(piece[0])):]

                if (temp_dict not in main_dict_list) and (bsc(temp_dict)) and (ssc(temp_dict, 1)):
                    main_dict_list.append(temp_dict)
        return main_dict_list


class Inflections(Derivational):
    _infNounDict = {}  # İsme eklenebilecek çekim ekleri sözlüğü
    _infVerbDict = {}  # Fiile eklenebilecek çekim ekleri sözlüğü

    def __init__(self):
        super().__init__()
        self.__load_inf_dict()

    def __dir__(self):
        return self.__dict__.keys()

    @classmethod
    def __load_inf_dict(cls) -> None:
        if not cls._infNounDict:
            cls._infNounDict = decompress_pickle(package_path() + 'data/infNoun.pbz2')
        if not cls._infVerbDict:
            cls._infVerbDict = decompress_pickle(package_path() + 'data/infVerb.pbz2')

    @staticmethod
    def _current_type(table_no: int, row_no: int) -> str:
        """
        Bulunan ekin kelimeye eklendiğindeki kelime tipini bulur
        :param table_no: Ekin tablo numarası
        :param row_no: Ekin satır numarası
        :return: isim, fiil vb. şekilde kelime tipini döndürür.
        """
        if table_no == 1:
            if 58 > row_no > 54:
                return 'isim,zarf'
            elif 34 < row_no < 55:
                return 'fiil'
            else:
                return 'isim'
        elif table_no == 2:
            if 62 > row_no > 59:
                return 'isim,zarf'
            else:
                return 'fiil'

    def _ifn(self, abc: str) -> list or None:
        """
        Arta kalan eki isme eklenebilen çekin ekleri(inf_noun)
        sözlüğü içerisinde arar ve sonucu döndürür.
        :param abc: aranacak arta kalan ek
        :return: sözlükte bulunan sonuçları liste halinde döndürür.
        """
        if abc in self._infNounDict:
            return [a1 for a1 in self._infNounDict[abc]]
        else:
            return None

    def _ifv(self, abc: str) -> list or None:
        """
        Arta kalan eki fiile eklenebilen çekin ekleri(inf_verb)
        sözlüğü içerisinde arar ve sonucu döndürür.
        :param abc: aranacak arta kalan ek
        :return: sözlükte bulunan sonuçları liste halinde döndürür.
        """
        if abc in self._infVerbDict:
            return [a1 for a1 in self._infVerbDict[abc]]
        else:
            return None

    def _inflection_lr(self, base_list=None) -> list:
        """
        Soldan sağa kelimeye eklenmiş olan olası çekim eklerini arar.
        :return: Liste içerisinde sözlük olarak sonuç döndürür.
        """
        result = []

        if base_list is None:
            main_dict_list = self.derivational_lr()
        else:
            main_dict_list = base_list

        if not main_dict_list:
            return []

        for main_dict in main_dict_list:
            if not main_dict['residual']:
                if (main_dict not in result) and bsc(main_dict) and ssc(main_dict):
                    result.append(main_dict)
                continue

            if 'fiil' in main_dict['currentType'][-1]:
                available = self._ifv(main_dict['residual'])
            else:
                available = self._ifn(main_dict['residual'])

            if available is None:
                continue

            for x in available:
                temp_dict = dict(main_dict)
                add_type = [self._current_type(j[0], j[1]) for j in x[1]]
                temp_dict['currentType'] = temp_dict['currentType'] + add_type
                temp_dict['suffixes'] = main_dict['suffixes'] + x[0]
                temp_dict['suffixPlace'] = main_dict['suffixPlace'] + x[1]
                temp_dict['suffixTypes'] = main_dict['suffixTypes'] + x[2]
                temp_dict['suffixProp'] = main_dict['suffixProp'] + x[3]
                temp_dict['residual'] = ''

                if temp_dict['baseType'] == ['kısaltma']:
                    temp_dict = abbr_control(temp_dict)
                    if temp_dict:
                        [result.append(tmp) for tmp in temp_dict if tmp not in result]
                else:
                    if temp_dict not in result:
                        src_ = general_control(temp_dict)
                        [result.append(x) for x in src_ if x not in result]
        return result

    def inflection_lr(self) -> list:
        return self._inflection_lr()

    def inflection_rl(self) -> list:
        result = []
        if "'" in self._orgWord:
            splited_word = [0, repc(to_lower(self._orgWord.split("'")[0]))]
        else:
            splited_word = self._splitter(self._word)

        temp_dict = {'base'        : "",
                     'verifiedBase': "",
                     'baseType'    : [],
                     'baseProp'    : ['0'],
                     'purview'     : '0',
                     'etymon'      : 'Türkçe',
                     'event'       : 0,
                     'currentType' : [],
                     'orgWord'     : self._orgWord,
                     'word'        : self._word,
                     'suffixes'    : [],
                     'suffixPlace' : [],
                     'suffixTypes' : [],
                     'suffixProp'  : [],
                     'residual'    : ""}

        for root in splited_word[1:]:
            residual = self._word[len(root):]
            temp_dict['base'] = root
            temp_dict['verifiedBase'] = root
            temp_dict['residual'] = residual

            if ("'" in self._orgWord) or (isCap(self._orgWord)):
                temp_dict['baseType'] = ['özel,kısaltma']
                temp_dict['currentType'] = ['özel,kısaltma']
                result = result + self._inflection_lr([temp_dict])

            temp_dict['baseType'] = ['fiil']
            temp_dict['currentType'] = ['fiil']
            result = result + self._inflection_lr([temp_dict])

            temp_dict['baseType'] = ['isim']
            temp_dict['currentType'] = ['isim']
            result = result + self._inflection_lr([temp_dict])

        if not result:
            temp_dict['base'] = self._word
            temp_dict['verifiedBase'] = self._word
            temp_dict['baseType'] = ['isim']
            temp_dict['currentType'] = ['isim']
            temp_dict['residual'] = ""
            result.append(temp_dict)
        return result


class TrnlpWord(Inflections):

    def __init__(self):
        super().__init__()
        self.__inf = []
        self.__morphology = {}
        self.__getBase = ""
        self.__getBaseType = ""
        self.__getStem = ""
        self.__getStemType = ""

    def __dir__(self):
        return self.__dict__.keys()

    def __str__(self):
        return writeable(self.__morphology)

    def setword(self, string_word: str) -> None:
        self.__morphology = {}
        self.__getBase = ""
        self.__getBaseType = ""
        self.__getStem = ""
        self.__getStemType = ""
        self._orgWord = string_word.strip()
        self._word = repa(repc(to_lower(string_word.strip())))
        self.__inf = self.inflection_lr()

        if self.__inf:
            self._arr_args()

    @property
    def get_base(self) -> str:
        return self.__getBase

    @property
    def get_base_type(self) -> str:
        return self.__getBaseType

    @property
    def get_stem(self) -> str:
        return self.__getStem

    @property
    def get_stem_type(self) -> str:
        return self.__getStemType

    @property
    def get_morphology(self) -> dict:
        return self.__morphology

    @property
    def get_inf(self) -> list:
        return self.__inf

    def _arr_args(self) -> None:
        self.__morphology = self._arr_infs()[0]
        self.__getBase = self.__morphology['verifiedBase']
        self.__getBaseType = ','.join(self.__morphology['baseType'])
        stem, stem_type = self._find_stem()
        self.__getStem = stem
        self.__getStemType = stem_type

    def _arr_infs(self) -> list:
        def nv_counter(arg):
            vcounter = 0
            ncounter = 0
            for i in range(len(arg['suffixPlace'])):
                a1, a2 = arg['suffixPlace'][i]
                st = arg['suffixTypes'][i]
                if a1 in {1, 4, 5}:
                    if st.startswith('Ef'):
                        ncounter += 1
                    else:
                        ncounter += 2
                else:
                    if a1 == 2:
                        vcounter += 2
                    else:
                        vcounter += 1
            return vcounter, ncounter

        temp1 = [x for x in self.__inf if 'kısaltma' in x['baseType']]
        temp2 = [x for x in self.__inf if 'özel' in x['baseType']]
        temp3 = [x for x in self.__inf if ('özel' not in x['baseType']) and ('kısaltma' not in x['baseType'])]
        if temp1:
            temp1.sort(key=lambda x: len(x['suffixes']))
        if temp2:
            temp2.sort(key=lambda x: len(x['suffixes']))
        if temp3:
            if "fiil" in temp3[0]['baseType']:
                temp3.sort(key=lambda x: nv_counter(x)[0])
            else:
                temp3.sort(key=lambda x: nv_counter(x)[1])
            temp3.reverse()
            temp3.sort(key=lambda x: len(x['suffixes']))

        if "." in self._orgWord:
            result = temp1 + temp3 + temp2
        elif "'" in self._orgWord:
            result = temp2 + temp1 + temp3
        elif isCap(self._orgWord):
            result = temp2 + temp3 + temp1
        else:
            result = temp3 + temp2 + temp1

        return result

    def _find_stem(self) -> tuple:
        last_list = list()
        morph = self.__morphology['base']
        current_type = self.__getBaseType
        i = -1
        for table, m_row in self.__morphology['suffixPlace']:
            i += 1
            if (table > 2) or (self.__morphology['suffixTypes'][i] in der_type):
                morph = morph + self.__morphology['suffixes'][i]
                current_type = self.__morphology['currentType'][i + 1]
                last_list = (table, m_row)
            elif table < 3:
                break

        if last_list:
            table, m_row = last_list
            if (5 in st_dict[table][m_row][5]) and (morph[-1] in terminal_devoicing):
                morph = morph[:-1] + terminal_devoicing[morph[-1]]
        else:
            morph = self.__morphology['verifiedBase']

        return morph, current_type

    def s_base(self, rlist=None) -> list:
        if not self.__inf:
            return []
        if rlist is None:
            rlist = self.__inf
        sbase_len = min(len(x['verifiedBase']) for x in rlist)
        return [x for x in rlist if len(x['verifiedBase']) == sbase_len]

    def l_base(self, rlist=None) -> list:
        if not self.__inf:
            return []
        if rlist is None:
            rlist = self.__inf
        lbase_len = max(len(x['verifiedBase']) for x in rlist)
        return [x for x in rlist if len(x['verifiedBase']) == lbase_len]

    def s_suffix(self, rlist=None) -> list:
        if not self.__inf:
            return []
        if rlist is None:
            rlist = self.__inf
        ssuffix_len = min(len(x['suffixes']) for x in rlist)
        return [x for x in rlist if len(x['suffixes']) == ssuffix_len]

    def l_suffix(self, rlist=None) -> list:
        if not self.__inf:
            return []
        if rlist is None:
            rlist = self.__inf
        lsuffix_len = max(len(x['suffixes']) for x in rlist)
        return [x for x in rlist if len(x['suffixes']) == lsuffix_len]

    def stemming(self):
        return self.inflection_rl()

    def is_negative(self) -> float:
        if not self.__inf:
            return False
        isneg = 0
        for x in self.__inf:
            if ('Olz' in x['suffixTypes']) or ('Ytsz' in x['suffixTypes']) or ((4, 4) in x['suffixPlace']) or (
                    'NEG' in x['baseProp']):
                isneg += 1
        if len(self.__inf) == 0:
            return 0
        return isneg / len(self.__inf)

    def is_plural(self) -> float:
        if not self.__inf:
            return 0
        isplu = 0
        for x in self.__inf:
            if 'TPL' in x['baseProp']:
                isplu += 1
                continue
            for _prop in x['suffixProp']:
                if 13 in _prop:
                    isplu += 1
                    break
        return isplu / len(self.__inf)

    def spelling(self) -> list:
        return syllabification(re.sub(r"\W", "", self._orgWord))

    def correct_form(self, arg=None):
        if arg is None:
            bdict = self.__morphology
        else:
            bdict = arg

        if not bdict:
            return ""

        if 'özel' in bdict['baseType']:
            if bdict['suffixes']:
                appl = bdict['verifiedBase']
                for i in range(len(bdict['suffixes'])):
                    if ("Ye-" in bdict['suffixTypes'][i]) or (bdict['suffixPlace'][i][0] > 2):
                        appl = appl + bdict['suffixes'][i]
                    else:
                        appl = appl + "'" + "".join(bdict['suffixes'][i:])
                        break
                return appl
            else:
                return bdict['verifiedBase']
        elif 'kısaltma' in bdict['baseType']:
            if 'SN' in bdict['baseProp']:
                return "{}{}".format(bdict['verifiedBase'], ''.join(bdict['suffixes']))
            else:
                return "{}'{}".format(bdict['verifiedBase'], ''.join(bdict['suffixes']))
        else:
            if bdict['event'] == 0:
                return "{}{}".format(bdict['verifiedBase'], ''.join(bdict['suffixes']))
            else:
                return "{}{}".format(bdict['base'], ''.join(bdict['suffixes']))


if __name__ == '__main__':
    pass



#!/usr/local/bin/python
# -*- coding: utf-8 -*-

"""
This file is part of trnlp.

trnlp is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

trnlp is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with trnlp.  If not, see <https://www.gnu.org/licenses/>.
Copyright (c) 2016-2020, Esat Mahmut Bayol

The full license is in the file LICENSE.txt, distributed with this software.

Ses Olayları :
1. Ünlü Düşmesi:
    oğul-u > oğlu

2. Ünlü Daralması:
    Sonu düz-geniş ünlü ile biten sözcükler “-yor” ekini aldığında sözcüğün sonundaki
    “a” veya “e” sesi daralarak “ı, i, u, ü” seslerinden birine dönüşür:
    başla-yor > başlıyor

    “de-” ve “ye-” fiilleri “-yor” ekinden başka ekler aldığında da daralmaya uğrar:
    de-y-ecek > diyecek
    ye-y-ecek > yiyecek

3. Ünlü Türemesi:
    Ünsüzle biten bazı sözcükler “-cik” yapım ekini aldığında iki ünsüz arasında bir ünlü türer:
    bir-cik > biricik
    genç-cik > gencecik

4. Ünsüz Yumuşaması
    Sonu “p, ç, t, k” sert ünsüzleri ile biten sözcükler, ünlü ile başlayan bir ek aldığında sözcüğün
    sonundaki bu sert ünsüzler yumuşayarak “b, c, d, g, ğ” ye dönüşür:
    dolap-ı > dolab-ı

5. Ünsüz Düşmesi
    “k” ünsüzüyle biten bazı sözcüklere “-cik/-cek” eklerinden biri geldiğinde sözcük sonundaki “k” ünsüzü düşer:
    çabuk-cak > çabucak
    sıcak-cık > sıcacık
    ufak-cık > ufacık

6. Ünsüz Türemesi (Ünsüz İkizleşmesi)
    Dilimize yabancı dillerden gelen bazı sözcükler, ünlü ile başlayan bir ek aldıklarında ya da ünlüyle
    başlayan bir yardımcı fiille birleştiğinde sözcüğün sonundaki ünsüzden bir tane türer:
    zan et- > zannet-
    af eyle- > affeyle-
    hak-ı > hakkı

1. Ünlü Düşmesi: oğul-u > oğl-u - UDUS +
2. Ünlü Daralması: başla-yor > başlı-yor - UDAR-YOR, UDAR +
3. Ünlü Türemesi: bir-cik > bir-icik - UTUR +
4. Ünsüz Yumuşaması: dolap-ı > dolab-ı - UZYUM +
6. Ünsüz Düşmesi: çabuk-cak > çabu-cak - UZDUS
7. Ünsüz Türemesi (Ünsüz İkizleşmesi): hak-ı > hakkı - UZTUR
"""

from trnlp.generator.createSuffixPickle import create_suffix_pickle
from trnlp.controler import *
from trnlp.finder import *
from trnlp.helper import *
import os
import re

__all__ = ['create_pickles']

gen_dict = {}
find_in_dict = {}


def load_gen_lexicon(lpath):
    with open(lpath, 'r', encoding='utf-8') as fl:
        lexicon = [line.strip().split('\t') for line in fl]
    return lexicon


def save_gen_lexicon(spath):
    compressed_pickle(spath, gen_dict)


def delete_pickle(lpath):
    delete_file_list = ['gen_tr_lex.pbz2', 'prop_tr_lex.pbz2', 'short_tr_lex.pbz2']
    file_list = os.listdir(lpath)
    [os.unlink('{}{}'.format(lpath, fileName)) for fileName in file_list if fileName in delete_file_list]
    print('.lexicons/ klasöründeki *.pickle dosyaları silindi!')


def find_etymon(etm):
    if etm == '0':
        return 'Türkçe'
    else:
        return etm


def arr_line(line):
    result = []
    word = to_lower(line[0])
    word_type = line[1].split(',')
    word_prop = line[2].split(',')
    word_acoustic = acoustic_phenomenon(word, word_prop)

    c_data = {'base'        : word,
              'verifiedBase': base_finder(line[5]),
              'baseType'    : word_type,
              'baseProp'    : word_prop,
              'etymon'      : find_etymon(line[4]),
              'event'       : 0,
              'currentType' : [line[1]]}

    if word_type == ['özel']:
        c_data['base'] = to_lower(base_finder(line[5]))
        c_data['verifiedBase'] = capital_tr(to_lower(c_data['verifiedBase']))
        c_data['purview'] = line[3]
        result.append((word, c_data))
        return result
    elif word_type == ['kısaltma']:
        c_data['verifiedBase'] = line[4]
        c_data['desc'] = line[3]
        result.append((re.sub(r"\W", "", word), c_data))
        return result
    else:
        ver_base = base_finder(line[5])
        ver_base_type = rfind_parenthesize(line[5]).split(',')
        c_data['purview'] = line[3]

        if len(word) == len(to_lower(ver_base)):
            result.append((word, c_data))
            if word_acoustic:
                for aco_word, aco_wordprop in word_acoustic:
                    copy_c_data = dict(c_data)
                    copy_c_data['base'] = aco_word
                    copy_c_data['baseProp'] = aco_wordprop
                    copy_c_data['event'] = 1
                    result.append((aco_word, copy_c_data))
        else:
            if word.startswith(to_lower(ver_base)):
                base_row = find_in_dict[(ver_base, ','.join(ver_base_type))]
                c_data['base'] = to_lower(ver_base)
                c_data['verifiedBase'] = ver_base
                c_data['baseType'] = ver_base_type
                c_data['baseProp'] = base_row[2].split(',')
                c_data['purview'] = base_row[3]
                c_data['etymon'] = find_etymon(base_row[4])
                c_data['event'] = 0
                c_data['currentType'] = [','.join(ver_base_type)]
                result.append((word, c_data))
                if word_acoustic:
                    for aco_word, aco_wordprop in word_acoustic:
                        copy_c_data = dict(c_data)
                        result.append((aco_word, copy_c_data))
            else:
                base_row = find_in_dict[(to_lower(ver_base), ','.join(ver_base_type))]
                base_prop = base_row[2].split(',')
                base_acoustic = acoustic_phenomenon(to_lower(ver_base), base_prop)
                for aco_base, aco_baseprop in base_acoustic:
                    if word.startswith(aco_base):
                        c_data['base'] = aco_base
                        c_data['verifiedBase'] = ver_base
                        c_data['baseType'] = ver_base_type
                        c_data['baseProp'] = aco_baseprop
                        c_data['purview'] = base_row[3]
                        c_data['etymon'] = find_etymon(base_row[4])
                        c_data['event'] = 1
                        c_data['currentType'] = [','.join(ver_base_type)]
                        result.append((word, c_data))
                    if word_acoustic:
                        for aco_word, aco_wordprop in word_acoustic:
                            copy_c_data = dict(c_data)
                            result.append((aco_word, copy_c_data))
    return result


def apply_to_dict(app_dict):
    for base, v in app_dict:
        base = repc(base)
        if base in gen_dict:
            if v not in gen_dict[base]:
                gen_dict[base].append(v)
        else:
            gen_dict[base] = [v]


def create_files(lex):
    global find_in_dict, gen_dict
    find_in_dict = {}
    gen_dict = {}
    for trline in lex:
        find_in_dict[(trline[0], trline[1])] = trline

    for line in lex:
        app_dict = arr_line(line)
        apply_to_dict(app_dict)


def create_pickles():
    p = package_path()

    delete_pickle(p + 'data/')

    lexicons = ((p + 'data/ana_sozluk.txt', p + 'data/gen_tr_lex'),
                (p + 'data/ozel_sozluk.txt', p + 'data/prop_tr_lex'),
                (p + 'data/kisaltma_sozluk.txt', p + 'data/short_tr_lex'))

    for lpath, spath in lexicons:
        trlexicon = load_gen_lexicon(lpath)
        print(lpath, 'dosyası analiz ediliyor...')
        create_files(trlexicon)
        save_gen_lexicon(spath)
        print(spath, 'dosyası başarı ile oluşturuldu.')

    create_suffix_pickle()


if __name__ == '__main__':
    create_pickles()
    del gen_dict



#!/usr/local/bin/python
# -*- coding: utf-8 -*-

"""
This file is part of trnlp.

trnlp is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

trnlp is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with trnlp.  If not, see <https://www.gnu.org/licenses/>.
Copyright (c) 2016-2020, Esat Mahmut Bayol

The full license is in the file LICENSE.txt, distributed with this software.
"""

from trnlp.finder import first_vowel, last_vowel
from trnlp.helper import vowel_harmony, word_to_number, wordtoten
from trnlp.data.suffix_tables import *
from itertools import product
from trnlp.constant import *
from trnlp.cleaner import *
import re

__all__ = ['bsc',
           'ssc',
           'abbr_control',
           'general_control',
           'acoustic_phenomenon']


def sfx_vowel_harmony(suffix: str, sfx_prop: tuple) -> bool:
    """
    :param suffix : Sesli uyumunun kontrol edileceği ek
    :param sfx_prop : Sesli uyumunun kontrol edileceği ekin özellik dizisi
    :return: 'True' ya da 'False' döndürür
    """
    if (7 in sfx_prop) or (vowel_harmony(suffix)):
        return True
    return False


def regex_solver(sfx_regex: str, sfx_prop: tuple) -> list:
    """
    :param sfx_regex : Ek tablosundaki ekin regex ifadesi
    :param sfx_prop : Ek tablosundaki ekin özellikler dizisi
    :return: Tüm ihtimalleri liste olarak döndürür
    """
    b = list(re.finditer(r"\w|\[(\w+)\]", sfx_regex))
    result = []
    for match in b:
        if '[' not in match.group():
            result.append(match.group())
        else:
            result.append(list(match.group()[1:-1]))
    return [''.join(x) for x in product(*result) if sfx_vowel_harmony(''.join(x), sfx_prop)]


def abbr_control(cabbr):
    copy_dict = dict(cabbr)
    abbr_prop = cabbr['baseProp']
    # print('desc', cabbr['desc'], cabbr)
    abbr_base_split = cabbr['desc'].split(';')
    abbrs = []
    if ('HK' in abbr_prop) or ('SN' in abbr_prop) or ('IHB' in abbr_prop):
        # print('abbr1 - ',abbr_base_split)
        for abbr_base in abbr_base_split:
            copy_dict['base'] = abbr_base
            if bsc(copy_dict) and ssc(copy_dict):
                cabbr['desc'] = abbr_base
                abbrs.append(cabbr)
    elif 'HB' in abbr_prop:
        # print('abbr - HB')
        if 'KG' in abbr_prop:
            # print('abbr - KG')
            if bsc(cabbr) and ssc(cabbr):
                abbrs.append(cabbr)
        else:
            if cabbr['base'][-1] in lowerVowels:
                copy_dict['base'] = cabbr['base'][-1]
                if bsc(copy_dict) and ssc(copy_dict):
                    abbrs.append(cabbr)
            else:
                copy_dict['base'] = cabbr['base'][-1] + 'e'
                if bsc(copy_dict) and ssc(copy_dict):
                    abbrs.append(cabbr)
    return abbrs


def letter_harmony(base: str, first_sfx: str, first_sfx_prop: tuple, suffix_place: tuple) -> bool:
    """
    Kök ile ilk ek arasındaki ses olaylarını kontrol eder.
    :param base: Kelime kökü
    :param first_sfx: ilk ek
    :param first_sfx_prop: İlk ekin özellik listesi
    :param suffix_place: İlk tablodaki yeri
    :return: True ya da False döndürür
    """
    if 0 in first_sfx_prop:
        if base[-1] in allVowels:
            return False
    elif 1 in first_sfx_prop:
        if base[-1] not in allVowels:
            return False

    if suffix_place == (6, 9):
        if (base[-1] not in allVowels) and (base[-1] not in {'r', 'l'}):
            return False
    elif suffix_place == (6, 8):
        if (base[-1] not in allVowels) and (base[-1] != 'r'):
            return False
    elif suffix_place == (6, 10):
        if (base not in ('ye', 'de')) and (base[-1] in allVowels):
            wtbase = wordtoten(base)
            if wtbase.count("1") == 1:
                return False

    if 4 in first_sfx_prop:
        if (base[-1] in fortis) and (first_sfx[0] in prefix_lenis):
            return False
        elif (base[-1] not in fortis) and (first_sfx[0] not in prefix_lenis):
            return False
    return True


def base_uyz_control(base: str, f_s: str, suffix_place: tuple,
                     base_prop: tuple, fs_prop: tuple,
                     base_event: int, ver_base: str) -> bool:
    """
    Kök ile ilk ek arasındaki ses uyumunu kontrol eder.
    :param base: Kelime kökü
    :param f_s: ilk ek
    :param suffix_place: ilk ek
    :param base_prop: Kelime kökünün özellik listsi
    :param fs_prop: İlk ekin özellik listesi
    :param base_event: İlk ekin özellik listesi
    :param ver_base: İlk ekin özellik listesi
    :return: True ya da False döndürür
    """

    def hrmny(h_rule):
        scf = first_vowel(f_s)

        if scf:
            if scf in h_rule[blv]:
                return True
            else:
                return False
        else:
            return True

    if (base_event == 1) and ('UDUS' in base_prop):
        base = ver_base

    blv = last_vowel(base)
    if not blv:
        return True

    if 'UYZ' in base_prop:
        if 7 in fs_prop:
            return True
        else:
            return hrmny(incHarmonyRule)
    else:
        if 7 in fs_prop:
            if suffix_place in {(2, 3), (2, 68)}:
                return hrmny(harmonyRule)
            else:
                return True
        else:
            return hrmny(harmonyRule)


def base_gz_control(first_sfx_place: tuple, morph_prop: list, first_sfx: str) -> bool:
    """
    Köke geniş zaman ekleme kontrolü
    :param first_sfx_place: ilk ekin tablodaki yeri
    :param morph_prop: Kelime kökünün özellik listsi
    :param first_sfx: ilk ek
    :return: True ya da False döndürür
    """
    if first_sfx_place in ((2, 7), (2, 8)):
        if ('GZ[r]' in morph_prop) and (first_sfx != 'r'):
            return False
        elif ('GZ[ar]' in morph_prop) and (first_sfx != 'ar'):
            return False
        elif ('GZ[er]' in morph_prop) and (first_sfx != 'er'):
            return False
        elif ('GZ[ır]' in morph_prop) and (first_sfx != 'ır'):
            return False
        elif ('GZ[ir]' in morph_prop) and (first_sfx != 'ir'):
            return False
        elif ('GZ[ür]' in morph_prop) and (first_sfx != 'ür'):
            return False
    return True


def base_utur_control(first_sfx_place: tuple, base_prop: list, first_sfx: str) -> bool:
    """
    Ünlü türemesi kontrolü
    :param first_sfx_place: ilk ekin tablodaki yeri
    :param base_prop: Kelime kökünün özellik listsi
    :param first_sfx: ilk ek
    :return: True ya da False döndürür
    """
    if first_sfx_place == (4, 21):
        if ('UTUR[i]' in base_prop) and (first_sfx[0] != 'i'):
            return False
        elif ('UTUR[a]' in base_prop) and (first_sfx[0] != 'a'):
            return False
        elif ('UTUR[ı]' in base_prop) and (first_sfx[0] != 'ı'):
            return False
        elif ('UTUR[e]' in base_prop) and (first_sfx[0] != 'e'):
            return False
    return True


def base_ch_control(base_prop: tuple, first_sfx: str, suffix_place: tuple) -> bool:
    """
    Ses olayı geçirmiş kökün ilk eke uyum kontrolü
    :param base_prop: Kelime kökünün özellik listsi
    :param first_sfx: ilk ek
    :param suffix_place: eklerin tablo ve sütun değeri
    :return: True ya da False döndürür
    """
    if 'UZTUR' in base_prop:
        if first_sfx[0] not in allVowels:
            return False
    if 'UZYUM' in base_prop:
        if first_sfx[0] not in allVowels:
            return False
    if 'UDUS' in base_prop:
        if first_sfx[0] not in allVowels:
            return False
    if 'UDAR-YOR' in base_prop:
        if suffix_place not in {(2, 4), (2, 67)}:
            return False
    if 'UD' in base_prop:
        if first_sfx[0] in allVowels:
            return False
    if 'UZDUS' in base_prop:
        if suffix_place not in ((4, 6), (4, 28)):
            return False
    if 'UDAR' in base_prop:
        if suffix_place not in {(2, 4), (2, 5), (2, 11), (2, 36), (2, 37), (2, 40), (8, 1), (9, 9)}:
            return False
    return True


def base_control(base_prop: list, first_sfx: str, suffix_place: tuple) -> bool:
    """
    Ses olayı geçirmemiş kökün ilk eke uyum kontrolü
    :param base_prop: Kelime kökünün özellik listsi
    :param first_sfx: ilk ek
    :param suffix_place: eklerin tablo ve sütun değeri
    :return: True ya da False döndürür
    """
    # print(base_prop)
    if base_prop == ['0']:
        return True
    if 'UZTUR' in base_prop:
        if first_sfx[0] in allVowels:
            return False
    if 'UZYUM' in base_prop:
        if first_sfx[0] in allVowels:
            return False
    if 'UDUS' in base_prop:
        if first_sfx[0] in allVowels:
            return False
    if 'UDAR-YOR' in base_prop:
        if suffix_place in {(2, 4), (2, 67)}:
            return False
    if 'UD' in base_prop:
        if first_sfx[0] in allVowels:
            return False
    if 'UZDUS' in base_prop:
        if suffix_place in ((4, 6), (4, 28)):
            return False
    if 'UDAR' in base_prop:
        if suffix_place in {(2, 4), (2, 5), (2, 11), (2, 36), (2, 37), (2, 40)}:
            return False
    return True


def bsc(arg: dict) -> bool:
    """
    Kökün ilk eke uyum kontrolü
    :param arg: Kök ek sözlüğü
    :return: True ya da False döndürür
    """
    # print('-' * 15)
    # print('Kök-Ek Kontrolü :')
    # print('Kök:', arg['base'], '- Kök Durumu:', arg['event'], 'Ekler:', arg['suffixes'])
    suffixes = arg['suffixes']

    if not suffixes:
        # print('Hiç ek almamış!')
        # print('BS KONTROL  - True')
        return True
    else:
        suffix = suffixes[0]

    base_type = arg['baseType']

    if base_type == ['bağlaç']:
        # print('Kökü bağlaç!')
        # print('BS KONTROL  - False')
        return False

    base = arg['base']
    base_prop = arg['baseProp']
    base_event = arg['event']
    ver_base = arg['verifiedBase']
    suffix_place = arg['suffixPlace'][0]
    suffix_prop = arg['suffixProp'][0]

    if 12 in suffix_prop:
        if not word_to_number(ver_base):
            return False

    if not letter_harmony(base, suffix, suffix_prop, suffix_place):
        # print('letter_harmony  - False')
        return False

    # print('letter_harmony  - OK')

    if not base_uyz_control(base, suffix, suffix_place, base_prop, suffix_prop, base_event, ver_base):
        # print('base_uyz_control  - False')
        return False

    # print('base_uyz_control  - OK')

    # if 'özel' in base_type:
    #     # print('Kökü özel isim!')
    #     # print('BS KONTROL  - True')
    #     return True

    if base_event == 0:
        if not base_control(base_prop, suffix, suffix_place):
            # print('base_control  - False')
            return False
    else:
        if not base_ch_control(base_prop, suffix, suffix_place):
            # print('base_ch_control  - False')
            return False

    # print('base_control-base_ch_control  - OK')

    if 'fiil' in base_type:
        if not base_gz_control(suffix_place, base_prop, suffix):
            # print('base_gz_control  - False')
            return False
        # print('base_gz_control  - OK')

    # print('BS KONTROL  - True')

    return True


def ssc_prop_control(suffix_list, suffix_place, suffix_prop):
    len_suffix = len(suffix_list)

    for sfxIndex in range(1, len_suffix):
        f_s = suffix_list[sfxIndex - 1]
        s_s = suffix_list[sfxIndex]
        fs_place = suffix_place[sfxIndex - 1]
        ss_place = suffix_place[sfxIndex]
        ss_prop = suffix_prop[sfxIndex]

        if fs_place == ss_place:
            return False

        if ss_place == (6, 9):
            if (f_s[-1] not in allVowels) and (f_s[-1] not in {'r', 'l'}):
                return False
        elif ss_place == (6, 8):
            if (f_s[-1] not in allVowels) and (f_s[-1] != 'r'):
                return False

        if (fs_place in {(2, 67), (2, 68)}) and (ss_place in {(2, 44), (2, 46), (2, 48), (2, 53)}):
            f_s = 'yor'

        if 0 in ss_prop:
            if f_s[-1] in allVowels:
                # print('if 0 in ss_prop  - False')
                return False
        elif 1 in ss_prop:
            if f_s[-1] not in allVowels:
                # print('elif 1 in ss_prop  - False')
                return False

        fs_prop = suffix_prop[sfxIndex - 1]

        if 9 in fs_prop:
            if fs_place[0] != ss_place[0]:
                # print('if 9 in fs_prop  - False')
                return False

        if 8 in fs_prop:
            if ss_place != (2, 4):
                # print('if 8 in fs_prop  - False')
                return False

        if 4 in ss_prop:
            if (f_s[-1] in fortis) and (s_s[0] in prefix_lenis):
                # print('if 4 in ss_prop  - False')
                return False
            if (s_s[0] not in prefix_lenis) and (f_s[-1] not in fortis):
                # print('if 4 in ss_prop  - False')
                return False

        if 5 in fs_prop:
            if (s_s[0] in allVowels) and (f_s[-1] not in 'bcdgğ'):
                # print('if 5 in fs_prop  - False')
                return False
            elif (f_s[-1] in 'bcdgğ') and (s_s[0] not in allVowels):
                # print('if 5 in fs_prop  - False')
                return False

        if 10 in fs_prop:
            if ss_place == (2, 4):
                # print('if 10 in fs_prop  - False')
                return False
    return True


def ssc7_control(base, base_prop, suffix_prop, suffix_list):
    sum_suffix = ""
    if ('UYZ' not in base_prop) and (7 not in suffix_prop[0]):
        sum_suffix = last_vowel(base)

    for i, sfxPrp in enumerate(suffix_prop, 0):
        if suffix_list[i] == 'giller':
            if not vowel_harmony(sum_suffix):
                return False
            sum_suffix = 'e'
            continue
        clns = clean_quites(suffix_list[i])
        if 7 in sfxPrp:
            if len(clns) > 1:
                sum_suffix = '{}{}'.format(sum_suffix, clns[0])
                if not vowel_harmony(sum_suffix):
                    return False
            else:
                if not vowel_harmony(sum_suffix):
                    return False
            sum_suffix = clns[-1]
        else:
            sum_suffix = '{}{}'.format(sum_suffix, clns)

    if not vowel_harmony(sum_suffix):
        return False

    return True


def ssc(infdaf: dict, id_status=2) -> bool:
    """
    Tüm eklerin uyum kontrolü
    :param infdaf: Kök ek sözlüğü
    :param id_status: ek türetme için gelen 0, yapım eki için gelenler 1, son hali için gelenler 2
    :return: True ya da False döndürür
    """
    suffix_list = infdaf['suffixes']
    # print('-' * 15)
    # print('Ek-Ek Kontrolü:')
    if not suffix_list:
        # print('Hiç ek almamış!')
        # print('SS KONTROL  - True')
        return True

    suffix_prop = infdaf['suffixProp']

    if id_status == 0:
        if 9 in suffix_prop[-1]:
            # print('id=0 - 9 in  - False')
            return False
    elif id_status == 1:
        if 9 in suffix_prop[-1]:
            # print('id=1 - 9 in  - False')
            return False
    elif id_status == 2:
        if (8 in suffix_prop[-1]) or (9 in suffix_prop[-1]):
            # print('id=2 - 8 or 9 in  - False')
            return False
        if (5 in suffix_prop[-1]) and (suffix_list[-1][-1] in 'bcdğ'):
            # print('id=2 - 5 in  - False')
            return False
        if 11 in suffix_prop[-1]:
            # print('id=2 - 11 in  - False')
            return False

    if len(suffix_list) < 2:
        # print('len(suffix_list) < 2')
        # print('SS KONTROL  - True')
        return True

    suffix_types = infdaf['suffixTypes']

    olz_count = suffix_types.count('Olz')
    ytsz_count = suffix_types.count('Ytsz')

    # if olz_count > 1:
    #     # print('olz_count > 1  - False')
    #     return False

    if olz_count + ytsz_count > 2:
        # print('olz_count + ytsz_count > 2  - False')
        return False

    suffix_place = infdaf['suffixPlace']

    if ((5, 2) in suffix_place):
        if olz_count > 0:
            if suffix_types.index('Olz') < suffix_place.index((5, 2)):
                return False
        elif ytsz_count > 0:
            if suffix_types.index('Ytsz') < suffix_place.index((5, 2)):
                return False

    if not elimination(suffix_place):
        # print('not elimination  - False')
        return False

    if not ssc_prop_control(suffix_list, suffix_place, suffix_prop):
        # print('ssc_prop_control  - False')
        return False

    if id_status != 0:
        base_prop = infdaf['baseProp']
        if 'UDUS' in base_prop:
            base = infdaf['verifiedBase']
        else:
            base = infdaf['base']

        if not ssc7_control(base, base_prop, suffix_prop, suffix_list):
            # print('ssc7_control  - False')
            return False
    else:
        if not ssc7_control('', ['UYZ'], suffix_prop, suffix_list):
            # print('ssc7_control  - False')
            return False

    # print('SS KONTROL  - True')

    return True


def elimination(suffix_place: list) -> bool:
    """
    Yanlış eklenmiş eklerin bir kısmını temizler
    :param suffix_place: Eklerin tablo, satır listesi
    :return: True ya da False döndürür
    """
    exclude = [[(7, 1), (5, 3)],
               [(5, 2), (8, 12)],
               [(5, 2), (6, 7)],
               [(6, 3), (6, 9)],
               [(6, 6), (6, 9)],
               [(4, 2), (5, 3)],
               [(7, 3), (5, 3)],
               [(7, 15), (5, 3)],
               [(6, 2), (6, 9)],
               [(4, 5), (4, 17)],
               [(5, 3), (8, 6)],
               [(5, 3), (6, 8)],
               [(6, 4), (8, 6)],
               [(6, 4), (6, 8)],
               [(5, 5), (6, 8)],
               [(5, 5), (6, 9)],
               [(8, 2), (5, 2)],
               [(5, 5), (7, 13)],
               [(4, 3), (5, 3)],
               [(6, 4), (7, 13)],
               [(5, 3), (7, 13)],
               [(5, 3), (7, 12)],
               [(5, 6), (7, 12)],
               ]

    suffix_place = [suffix_place[i - 2:i] for i in range(2, len(suffix_place) + 1)]
    if any(x in suffix_place for x in exclude):
        return False
    return True


def uzyum(string: str):
    yum = {'ç': 'c', 'p': 'b', 't': 'd', 'k': 'ğ', 'g': 'ğ'}
    if string.endswith('nk'):
        return string[:-1] + 'g'
    else:
        return string[:-1] + yum[string[-1]]


def uztur(string: str):
    return string + string[-1]


def udus(string: str):
    return string[:-2] + string[-1]


def udar(string: str):
    udardict = {'e' : 'i',
                'üe': 'ü',
                'ua': 'u',
                'ee': 'i',
                'aa': 'ı',
                'öe': 'ü',
                'ıa': 'ı',
                'oa': 'u',
                'ie': 'i',
                'ae': 'i',
                'âa': 'ı',
                'oe': 'ü',
                'ue': 'ü'}
    clean = clean_quites(string)
    return string[:-1] + udardict[clean[-2:]]


def uzdus(string: str):
    return string[:-1]


def acoustic_phenomenon(word, wordprop):
    if 'UZYUM' in wordprop:
        if 'UDUS' in wordprop:
            return [(uzyum(udus(word)), wordprop)]
        elif 'UZTUR' in wordprop:
            return [(uztur(uzyum(word)), wordprop)]
        elif 'UZDUS' in wordprop:
            uzyum_prop = [x for x in wordprop if x != 'UZDUS']
            uzdus_prop = [x for x in wordprop if x != 'UZYUM']
            return [(uzyum(word), uzyum_prop), (uzdus(word), uzdus_prop)]
        else:
            return [(uzyum(word), wordprop)]
    elif ('UDAR-YOR' in wordprop) or ('UDAR' in wordprop):
        return [(udar(word), wordprop)]
    elif 'UZTUR' in wordprop:
        return [(uztur(word), wordprop)]
    elif 'UZDUS' in wordprop:
        return [(uzdus(word), wordprop)]
    elif 'UDUS' in wordprop:
        return [(udus(word), wordprop)]
    elif 'UD' in wordprop:
        return [(uzdus(word), wordprop)]


def general_control(arg):
    result = []

    if 'bağlaç' in arg['baseType']:
        return result

    places = arg['suffixPlace']
    in_corrector = [xi for xi in places if xi in corrector_tuple]

    if arg['event'] == 0:
        acoustic = acoustic_phenomenon(arg['base'], arg['baseProp'])
    else:
        acoustic = [(arg['base'], arg['baseProp'])]

    if in_corrector:
        suffixes = []
        suffix_place = []
        suffix_types = []
        suffix_prop = []
        addable = False
        for i, place in enumerate(arg['suffixPlace'], 0):
            if place in corrector_data:
                addable = True
            if addable:
                sp = corrector_data[place]
                s_prop = st_dict[sp[0]][sp[1]][5]
                suffixes.append(regex_solver(st_dict[sp[0]][sp[1]][3], s_prop))
                suffix_place.append(sp)
                suffix_types.append(st_dict[sp[0]][sp[1]][2])
                suffix_prop.append(s_prop)
                addable = False
            else:
                suffixes.append([arg['suffixes'][i]])
                suffix_place.append(place)
                suffix_types.append(arg['suffixTypes'][i])
                suffix_prop.append(arg['suffixProp'][i])

        arg['suffixPlace'] = suffix_place
        arg['suffixTypes'] = suffix_types
        arg['suffixProp'] = suffix_prop

        pro_suffixes = [list(x) for x in product(*suffixes)]
        # print(pro_suffixes)

        for sfx in pro_suffixes:
            ctemp = dict(arg)
            ctemp['suffixes'] = sfx
            if bsc(ctemp) and ssc(ctemp):
                ctemp['word'] = ctemp['base'] + ''.join(ctemp['suffixes'])
                result.append(ctemp)
        if (not result) and (acoustic):
            for base, base_prop in acoustic:
                for sfx in pro_suffixes:
                    ctemp = dict(arg)
                    ctemp['base'] = base
                    ctemp['baseProp'] = base_prop
                    ctemp['event'] = 1
                    ctemp['suffixes'] = sfx
                    if bsc(ctemp) and ssc(ctemp):
                        ctemp['word'] = ctemp['base'] + ''.join(ctemp['suffixes'])
                        result.append(ctemp)
    else:
        if bsc(arg) and ssc(arg):
            result.append(arg)
    return result


if __name__ == '__main__':
    pass



from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
        name='trnlp',
        version='0.2.2a0',
        description='Türkçe doğal dil işleme araçları',
        long_description=long_description,
        long_description_content_type='text/markdown',
        url='https://github.com/brolin59/trnlp',
        download_url='https://github.com/brolin59/trnlp/archive/0.2.1a0.tar.gz',
        author='Esat Mahmut Bayol',
        author_email='trnlp2020@gmail.com',
        license='GNU General Public License v3 (GPLv3)',
        classifiers=[
                'Development Status :: 3 - Alpha',
                'Intended Audience :: Developers',
                'Intended Audience :: Education',
                'Intended Audience :: Science/Research',
                'Intended Audience :: Information Technology',
                'Environment :: Win32 (MS Windows)',
                'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
                'Programming Language :: Python :: 3.7',
        ],
        keywords=['türkçe', 'doğal', 'dil', 'işleme', 'nlp', 'ddi', 'trnlp'],
        packages=find_packages(),
        include_package_data=True,
        python_requires='>=3.5',
        project_urls={
                'Documentation': 'https://github.com/brolin59/trnlp/wiki',
                'Source'       : 'https://github.com/brolin59/trnlp/tree/master/trnlp',
        },
)



#!/usr/local/bin/python
# -*- coding: utf-8 -*-

"""
This file is part of trnlp.

trnlp is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

trnlp is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with trnlp.  If not, see <https://www.gnu.org/licenses/>.
Copyright (c) 2016-2020, Esat Mahmut Bayol

The full license is in the file LICENSE.txt, distributed with this software.
"""
from trnlp.helper import to_lower, n_gram
from trnlp.tokenization import word_token
from collections import Counter
import shutil
import os

__all__ = ['count_all_txt']


def open_txt(txt_file_path: str):
    try:
        with open(txt_file_path, 'r', encoding='utf-8') as fl:
            txt_file = fl.read()
        del fl
    except UnicodeDecodeError:
        with open(txt_file_path, 'r') as fl:
            txt_file = fl.read()
        del fl
    return txt_file


def huddle_txt_files(folder_path: str):
    def write_to_file(n_lines):
        trg_path = target_path + str(len(os.listdir(target_path)) + 1) + '.txt'
        with open(trg_path, "w", encoding="utf8") as wfl:
            for row in n_lines:
                wfl.write(row + '\n')

    target_path = folder_path + '/trnlp_temp/'

    print('{} geçici klasörü oluşturuluyor...'.format(target_path))

    if os.path.isdir(target_path):
        try:
            shutil.rmtree(target_path)
        except OSError as e:
            print("Error: %s : %s" % (target_path, e.strerror))

    os.makedirs(target_path)

    print('{} klasöründeki .txt dosyaları okunuyor...'.format(folder_path))

    rows = []
    for root, dirs, files in os.walk(folder_path):
        for file in files:
            if file[-4:] == '.txt':
                file_path = os.path.join(root, file)
                lines = open_txt(file_path)
                lines = lines.split('\n')
                for line in lines:
                    if line:
                        rows.append(line)

                    if len(rows) == 200000:
                        write_to_file(rows)
                        del rows
                        rows = []
                        continue
    if rows:
        write_to_file(rows)


def count_all_txt(folder_path, ngram=1) -> dict:
    if folder_path.endswith("/"):
        folder_path = folder_path[:-1]

    if os.path.isdir(folder_path):
        pass
    else:
        raise FileNotFoundError("{} klasörü bulunamadı!".format(folder_path))

    counting_word = {}
    counting_word = Counter(counting_word)

    huddle_txt_files(folder_path)

    for root, dirs, files in os.walk(folder_path + '/trnlp_temp/'):
        print('Toplam {} adet dosya bulundu...'.format(len(files)))
        all_txt = ""
        for file in files:
            all_txt = ""
            if file[-4:] == '.txt':
                file_path = os.path.join(root, file)
                lines = open_txt(file_path)
                all_txt = all_txt + lines + '\n'

            print('{} isimli dosya sayılıyor...'.format(file))

            all_txt = [to_lower(low) for low in word_token(all_txt)]

            if ngram == 1:
                counting_word = counting_word + Counter(all_txt)
            else:
                counting_word = counting_word + Counter(n_gram(all_txt, ngram))

        if all_txt:
            counting_word = counting_word + Counter(all_txt)

    counting_word = dict(counting_word)

    print('TÜM DOSYALAR SAYILDI!\n')
    print('{} geçici klasörü siliniyor...'.format(folder_path + '/trnlp_temp/'))

    if os.path.isdir(folder_path + '/trnlp_temp/'):
        try:
            shutil.rmtree(folder_path + '/trnlp_temp/')
        except OSError as e:
            print("Error: %s : %s" % (folder_path + '/trnlp_temp/', e.strerror))

    return counting_word


if __name__ == '__main__':
    pass
